# sage_setup: distribution = sagemath-flint
# distutils: libraries = flint
# distutils: depends = flint/fmpz_vec.h

################################################################################
# This file is auto-generated by the script
#   SAGE_ROOT/src/sage_setup/autogen/flint_autogen.py.
# From the commit 3e2c3a3e091106a25ca9c6fba28e02f2cbcd654a
# Do not modify by hand! Fix and rerun the script instead.
################################################################################

from libc.stdio cimport FILE
from sage.libs.gmp.types cimport *
from sage.libs.mpfr.types cimport *
from sage.libs.flint.types cimport *

cdef extern from "flint_wrap.h":
    fmpz * _fmpz_vec_init(slong len) noexcept
    void _fmpz_vec_clear(fmpz * vec, slong len) noexcept
    void _fmpz_vec_randtest(fmpz * f, flint_rand_t state, slong len, flint_bitcnt_t bits) noexcept
    void _fmpz_vec_randtest_unsigned(fmpz * f, flint_rand_t state, slong len, flint_bitcnt_t bits) noexcept
    slong _fmpz_vec_max_bits(const fmpz * vec, slong len) noexcept
    slong _fmpz_vec_max_bits_ref(const fmpz * vec, slong len) noexcept
    void _fmpz_vec_sum_max_bits(slong * sumabs, slong * maxabs, const fmpz * vec, slong len) noexcept
    mp_size_t _fmpz_vec_max_limbs(const fmpz * vec, slong len) noexcept
    void _fmpz_vec_height(fmpz_t height, const fmpz * vec, slong len) noexcept
    slong _fmpz_vec_height_index(const fmpz * vec, slong len) noexcept
    int _fmpz_vec_fread(FILE * file, fmpz ** vec, slong * len) noexcept
    int _fmpz_vec_read(fmpz ** vec, slong * len) noexcept
    int _fmpz_vec_fprint(FILE * file, const fmpz * vec, slong len) noexcept
    int _fmpz_vec_print(const fmpz * vec, slong len) noexcept
    void _fmpz_vec_get_nmod_vec(mp_ptr res, const fmpz * poly, slong len, nmod_t mod) noexcept
    void _fmpz_vec_set_nmod_vec(fmpz * res, mp_srcptr poly, slong len, nmod_t mod) noexcept
    void _fmpz_vec_get_fft(mp_limb_t ** coeffs_f, const fmpz * coeffs_m, slong l, slong length) noexcept
    void _fmpz_vec_set_fft(fmpz * coeffs_m, slong length, const mp_ptr * coeffs_f, slong limbs, slong sign) noexcept
    slong _fmpz_vec_get_d_vec_2exp(double * appv, const fmpz * vec, slong len) noexcept
    void _fmpz_vec_set(fmpz * vec1, const fmpz * vec2, slong len2) noexcept
    void _fmpz_vec_swap(fmpz * vec1, fmpz * vec2, slong len2) noexcept
    void _fmpz_vec_zero(fmpz * vec, slong len) noexcept
    void _fmpz_vec_neg(fmpz * vec1, const fmpz * vec2, slong len2) noexcept
    void _fmpz_vec_scalar_abs(fmpz * vec1, const fmpz * vec2, slong len2) noexcept
    bint _fmpz_vec_equal(const fmpz * vec1, const fmpz * vec2, slong len) noexcept
    bint _fmpz_vec_is_zero(const fmpz * vec, slong len) noexcept
    void _fmpz_vec_max(fmpz * vec1, const fmpz * vec2, const fmpz * vec3, slong len) noexcept
    void _fmpz_vec_max_inplace(fmpz * vec1, const fmpz * vec2, slong len) noexcept
    void _fmpz_vec_sort(fmpz * vec, slong len) noexcept
    void _fmpz_vec_add(fmpz * res, const fmpz * vec1, const fmpz * vec2, slong len2) noexcept
    void _fmpz_vec_sub(fmpz * res, const fmpz * vec1, const fmpz * vec2, slong len2) noexcept
    void _fmpz_vec_scalar_mul_fmpz(fmpz * vec1, const fmpz * vec2, slong len2, const fmpz_t x) noexcept
    void _fmpz_vec_scalar_mul_si(fmpz * vec1, const fmpz * vec2, slong len2, slong c) noexcept
    void _fmpz_vec_scalar_mul_ui(fmpz * vec1, const fmpz * vec2, slong len2, ulong c) noexcept
    void _fmpz_vec_scalar_mul_2exp(fmpz * vec1, const fmpz * vec2, slong len2, ulong exp) noexcept
    void _fmpz_vec_scalar_divexact_fmpz(fmpz * vec1, const fmpz * vec2, slong len2, const fmpz_t x) noexcept
    void _fmpz_vec_scalar_divexact_si(fmpz * vec1, const fmpz * vec2, slong len2, slong c) noexcept
    void _fmpz_vec_scalar_divexact_ui(fmpz * vec1, const fmpz * vec2, slong len2, ulong c) noexcept
    void _fmpz_vec_scalar_fdiv_q_fmpz(fmpz * vec1, const fmpz * vec2, slong len2, const fmpz_t c) noexcept
    void _fmpz_vec_scalar_fdiv_q_si(fmpz * vec1, const fmpz * vec2, slong len2, slong c) noexcept
    void _fmpz_vec_scalar_fdiv_q_ui(fmpz * vec1, const fmpz * vec2, slong len2, ulong c) noexcept
    void _fmpz_vec_scalar_fdiv_q_2exp(fmpz * vec1, const fmpz * vec2, slong len2, ulong exp) noexcept
    void _fmpz_vec_scalar_fdiv_r_2exp(fmpz * vec1, const fmpz * vec2, slong len2, ulong exp) noexcept
    void _fmpz_vec_scalar_tdiv_q_fmpz(fmpz * vec1, const fmpz * vec2, slong len2, const fmpz_t c) noexcept
    void _fmpz_vec_scalar_tdiv_q_si(fmpz * vec1, const fmpz * vec2, slong len2, slong c) noexcept
    void _fmpz_vec_scalar_tdiv_q_ui(fmpz * vec1, const fmpz * vec2, slong len2, ulong c) noexcept
    void _fmpz_vec_scalar_tdiv_q_2exp(fmpz * vec1, const fmpz * vec2, slong len2, ulong exp) noexcept
    void _fmpz_vec_scalar_addmul_si(fmpz * vec1, const fmpz * vec2, slong len2, slong c) noexcept
    void _fmpz_vec_scalar_addmul_ui(fmpz * vec1, const fmpz * vec2, slong len2, ulong c) noexcept
    void _fmpz_vec_scalar_addmul_fmpz(fmpz * vec1, const fmpz * vec2, slong len2, const fmpz_t c) noexcept
    void _fmpz_vec_scalar_addmul_si_2exp(fmpz * vec1, const fmpz * vec2, slong len2, slong c, ulong exp) noexcept
    void _fmpz_vec_scalar_submul_fmpz(fmpz * vec1, const fmpz * vec2, slong len2, const fmpz_t x) noexcept
    void _fmpz_vec_scalar_submul_si(fmpz * vec1, const fmpz * vec2, slong len2, slong c) noexcept
    void _fmpz_vec_scalar_submul_si_2exp(fmpz * vec1, const fmpz * vec2, slong len2, slong c, ulong e) noexcept
    void _fmpz_vec_sum(fmpz_t res, const fmpz * vec, slong len) noexcept
    void _fmpz_vec_prod(fmpz_t res, const fmpz * vec, slong len) noexcept
    void _fmpz_vec_scalar_mod_fmpz(fmpz * res, const fmpz * vec, slong len, const fmpz_t p) noexcept
    void _fmpz_vec_scalar_smod_fmpz(fmpz * res, const fmpz * vec, slong len, const fmpz_t p) noexcept
    void _fmpz_vec_content(fmpz_t res, const fmpz * vec, slong len) noexcept
    void _fmpz_vec_content_chained(fmpz_t res, const fmpz * vec, slong len, const fmpz_t input) noexcept
    void _fmpz_vec_lcm(fmpz_t res, const fmpz * vec, slong len) noexcept
    void _fmpz_vec_dot(fmpz_t res, const fmpz * vec1, const fmpz * vec2, slong len2) noexcept
    void _fmpz_vec_dot_ptr(fmpz_t res, const fmpz * vec1, fmpz ** const vec2, slong offset, slong len) noexcept



# sage_setup: distribution = sagemath-pari
r"""
Finite residue fields (PARI implementation)
"""

# *****************************************************************************
#       Copyright (C) 2007-2019 David Roe <roed@math.harvard.edu>
#                     2007      William Stein <wstein@gmail.com>
#                     2008      John Cremona
#                     2008      Robert Bradshaw
#                     2009      Nick Alexander
#                     2010      Robert L. Miller
#                     2010-2013 Simon King
#                     2010-2017 Jeroen Demeyer
#                     2012      Travis Scrimshaw
#                     2016-2021 Frédéric Chapoton
#                     2021-2022 Antonio Rojas
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
# *****************************************************************************

from sage.rings.finite_rings.finite_field_pari_ffelt import FiniteField_pari_ffelt
from sage.rings.finite_rings.residue_field import ResidueField_generic, ResidueFieldHomomorphism_global, ReductionMap


class ResidueFiniteField_pari_ffelt(ResidueField_generic, FiniteField_pari_ffelt):
    """
    The class representing residue fields of number fields that have non-prime
    order at least `2^16`.

    EXAMPLES::

        sage: # needs sage.rings.number_field
        sage: x = polygen(ZZ, 'x')
        sage: K.<a> = NumberField(x^3 - 7)
        sage: P = K.ideal(923478923).factor()[0][0]
        sage: k = K.residue_field(P)
        sage: k.degree()
        2
        sage: OK = K.maximal_order()
        sage: c = OK(a)
        sage: b = k(c)
        sage: b + c
        2*abar
        sage: b*c
        664346875*abar + 535606347
        sage: k.base_ring()
        Finite Field of size 923478923

        sage: R.<t> = GF(5)[]
        sage: P = R.ideal(4*t^12 + 3*t^11 + 4*t^10 + t^9 + t^8
        ....:             + 3*t^7 + 2*t^6 + 3*t^4 + t^3 + 3*t^2 + 2)
        sage: k.<a> = P.residue_field()
        sage: type(k)
        <class 'sage.rings.finite_rings.residue_field_pari_ffelt.ResidueFiniteField_pari_ffelt_with_category'>
        sage: k(1/t)
        3*a^11 + a^10 + 3*a^9 + 2*a^8 + 2*a^7 + a^6 + 4*a^5 + a^3 + 2*a^2 + a
    """

    def __init__(self, p, characteristic, name, modulus, to_vs, to_order, PB):
        """
        Initialize ``self``.

        EXAMPLES:

        We create a residue field with implementation ``pari_ffelt``::

            sage: # needs sage.rings.number_field
            sage: x = polygen(ZZ, 'x')
            sage: K.<a> = NumberField(x^3 - 7)
            sage: P = K.ideal(923478923).factor()[0][0]
            sage: type(P.residue_field())
            <class 'sage.rings.finite_rings.residue_field_pari_ffelt.ResidueFiniteField_pari_ffelt_with_category'>
        """
        ResidueField_generic.__init__(self, p)
        FiniteField_pari_ffelt.__init__(self, characteristic, modulus, name)
        K = OK = p.ring()
        if OK.is_field():
            OK = OK.ring_of_integers()
        else:
            K = K.fraction_field()
        if PB is None:
            PBinv = None
        else:
            PBinv = PB**(-1)
        self._populate_coercion_lists_(coerce_list=[self.base_ring(),
                                                    ResidueFieldHomomorphism_global(OK, self, to_vs, to_order, PB, PBinv)],
                                       convert_list=[ReductionMap(K, self, to_vs, to_order, PB, PBinv)])

    def _element_constructor_(self, x):
        """
        Coerce ``x`` into ``self``.

        EXAMPLES::

            sage: # needs sage.rings.number_field
            sage: x = polygen(ZZ, 'x')
            sage: K.<aa> = NumberField(x^3 - 2)
            sage: P = K.factor(10007)[0][0]
            sage: P.residue_class_degree()
            2
            sage: ff.<alpha> = P.residue_field(); ff
            Residue field in alpha of Fractional ideal (12*aa^2 - 189*aa + 475)
            sage: type(ff)
            <class 'sage.rings.finite_rings.residue_field_pari_ffelt.ResidueFiniteField_pari_ffelt_with_category'>
            sage: ff(alpha^2 + 1)
            7521*alpha + 4131
            sage: ff(17/3)
            6677
            sage: V = ff.vector_space(map=False); v = V([3,-2])                         # needs sage.modules
            sage: type(ff.convert_map_from(V))                                          # needs sage.modules
            <class 'sage.structure.coerce_maps.DefaultConvertMap_unique'>
            sage: ff(v)  # indirect doctest                                             # needs sage.modules
            10005*alpha + 3

            sage: R.<t> = GF(5)[]; P = R.ideal(4*t^12 + 3*t^11 + 4*t^10 + t^9 + t^8 + 3*t^7 + 2*t^6 + 3*t^4 + t^3 + 3*t^2 + 2)
            sage: k.<a> = P.residue_field()
            sage: V = k.vector_space(map=False); v = V([1,2,3,4,5,6,7,8,9,0,1,2]); k(v)  # indirect doctest             # needs sage.modules
            2*a^11 + a^10 + 4*a^8 + 3*a^7 + 2*a^6 + a^5 + 4*a^3 + 3*a^2 + 2*a + 1
        """
        try:
            return self.element_class(self, x)
        except TypeError:
            return ResidueField_generic._element_constructor_(self, x)



# sage_setup: distribution = sagemath-modules
r"""
Morphisms of modules with a basis

This module contains a hierarchy of classes for morphisms of modules
with a basis (category :class:`Modules.WithBasis`):

- :class:`ModuleMorphism`
- :class:`ModuleMorphismByLinearity`
- :class:`ModuleMorphismFromMatrix`
- :class:`ModuleMorphismFromFunction`
- :class:`TriangularModuleMorphism`
- :class:`TriangularModuleMorphismByLinearity`
- :class:`TriangularModuleMorphismFromFunction`

These are internal classes; it is recommended *not* to use them
directly, and instead to construct morphisms through the
:meth:`ModulesWithBasis.ParentMethods.module_morphism` method of the
domain, or through the homset. See the former for an overview
of the possible arguments.

EXAMPLES:

We construct a morphism through the method
:meth:`ModulesWithBasis.ParentMethods.module_morphism`, by specifying
the image of each element of the distinguished basis::

    sage: X = CombinatorialFreeModule(QQ, [1,2,3]);   x = X.basis()
    sage: Y = CombinatorialFreeModule(QQ, [1,2,3,4]); y = Y.basis()
    sage: on_basis = lambda i: Y.monomial(i) + 2*Y.monomial(i+1)

    sage: phi1 = X.module_morphism(on_basis, codomain=Y)
    sage: phi1(x[1])
    B[1] + 2*B[2]

    sage: phi1
    Generic morphism:
      From: Free module generated by {1, 2, 3} over Rational Field
      To:   Free module generated by {1, 2, 3, 4} over Rational Field
    sage: phi1.parent()
    Set of Morphisms from Free module generated by {1, 2, 3} over Rational Field to Free module generated by {1, 2, 3, 4} over Rational Field in Category of finite dimensional vector spaces with basis over Rational Field
    sage: phi1.__class__
    <class 'sage.modules.with_basis.morphism.ModuleMorphismByLinearity_with_category'>

Constructing the same morphism from the homset::

    sage: H = Hom(X,Y)
    sage: phi2 = H(on_basis=on_basis)
    sage: phi1 == phi2
    True

Constructing the same morphism directly using the class; no backward
compatibility is guaranteed in this case::

    sage: from sage.modules.with_basis.morphism import ModuleMorphismByLinearity
    sage: phi3 = ModuleMorphismByLinearity(X, on_basis, codomain=Y)
    sage: phi3 == phi1
    True

.. WARNING::

    The hierarchy of classes implemented in this module is one of the
    first non-trivial hierarchies of classes for morphisms. It is
    hitting a couple scaling issues:

    - There are many independent properties from which module
      morphisms can get code (being defined by linearity, from a
      matrix, or a function; being triangular, being diagonal,
      ...). How to mitigate the class hierarchy growth?

      This will become even more stringent as more properties are
      added (e.g. being defined from generators for an algebra
      morphism, ...)

      Categories, whose primary purpose is to provide infrastructure
      for handling such large hierarchy of classes, can't help at this
      point: there is no category whose morphisms are triangular
      morphisms, and it's not clear such a category would be sensible.

    - How to properly handle ``__init__`` method calls and
      multiple inheritance?

    - Who should be in charge of setting the default category: the
      classes themselves, or
      :meth:`ModulesWithBasis.ParentMethods.module_morphism`?

    Because of this, the hierarchy of classes, and the specific APIs,
    is likely to be refactored as better infrastructure and best
    practices emerge.

AUTHORS:

- Nicolas M. Thiery (2008-2015)

- Jason Bandlow and Florent Hivert (2010): Triangular Morphisms

- Christian Stump (2010): :issue:`9648` module_morphism's to a wider class
  of codomains

Before :issue:`8678`, this hierarchy of classes used to be in
sage.categories.modules_with_basis; see :issue:`8678` for the complete log.
"""

# ****************************************************************************
#  Copyright (C) 2015 Nicolas M. Thiery <nthiery at users.sf.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.categories.fields import Fields
from sage.categories.modules import Modules
from sage.misc.call import attrcall
# The identity function would deserve a more canonical location
from sage.misc.c3_controlled import identity
from sage.categories.commutative_additive_semigroups import CommutativeAdditiveSemigroups
from sage.categories.homset import Hom
from sage.categories.modules_with_basis import ModulesWithBasis
from sage.categories.morphism import SetMorphism, Morphism
from sage.categories.sets_cat import Sets
from sage.categories.sets_with_partial_maps import SetsWithPartialMaps
from sage.structure.richcmp import op_EQ, op_NE
from sage.structure.element import Matrix
from sage.structure.sage_object import SageObject


class ModuleMorphism(Morphism):
    """
    The top abstract base class for module with basis morphisms.

    INPUT:

    - ``domain`` -- a parent in ``ModulesWithBasis(...)``
    - ``codomain`` -- a parent in ``Modules(...)``
    - ``category`` -- a category or ``None`` (default: ``None``)
    - ``affine`` -- whether we define an affine module morphism
      (default: ``False``)

    Construct a module morphism from ``domain`` to ``codomain`` in the
    category ``category``. By default, the category is the first of
    ``Modules(R).WithBasis().FiniteDimensional()``,
    ``Modules(R).WithBasis()``, ``Modules(R)``,
    ``CommutativeAdditiveMonoids()`` that contains both the domain and
    the codomain. If initializing an affine morphism, then `Sets()` is
    used instead.

    .. SEEALSO::

        - :meth:`ModulesWithBasis.ParentMethods.module_morphism` for
          usage information and examples;
        - :mod:`sage.modules.with_basis.morphism` for a technical
          overview of the classes for module morphisms;
        - :class:`ModuleMorphismFromFunction` and
          :class:`TriangularModuleMorphism`.

    The role of this class is minimal: it provides an :meth:`__init__`
    method which:

    - handles the choice of the default category
    - handles the proper inheritance from categories by updating the
      class of ``self`` upon construction.
    """
    def __init__(self, domain, codomain=None, category=None, affine=False):
        """
        Initialization of module morphisms.

        TESTS::

            sage: X = CombinatorialFreeModule(ZZ, [-2, -1, 1, 2])
            sage: Y = CombinatorialFreeModule(ZZ, [1, 2])
            sage: from sage.modules.with_basis.morphism import ModuleMorphismByLinearity
            sage: phi = ModuleMorphismByLinearity(X, on_basis=Y.monomial * abs)
            sage: TestSuite(phi).run()
        """
        if category is None:
            if domain not in ModulesWithBasis:
                raise ValueError("domain(=%s) should be a module with basis" % codomain)
            base_ring = domain.base_ring()

            if not hasattr(codomain, 'base_ring'):
                raise ValueError("codomain(=%s) needs to have a base_ring attribute" % codomain)
            # codomain should be a module over base_ring
            # The natural test would be ``codomains in Modules(base_ring)``
            # But this is not properly implemented yet::
            #
            #     sage: CC in Modules(QQ)
            #     False
            #     sage: QQ in Modules(QQ)
            #     False
            #     sage: CC[x] in Modules(QQ)
            #     False

            # The test below is a bit more restrictive
            if (not codomain.base_ring().has_coerce_map_from(base_ring)) \
               and (not codomain.has_coerce_map_from(base_ring)):
                raise ValueError("codomain(=%s) should be a module over the base ring of the domain(=%s)" % (codomain, domain))

            if affine:
                # We don't yet have a category whose morphisms are affine morphisms
                category = Sets()
            else:
                C = Modules(base_ring)
                for D in [C.WithBasis().FiniteDimensional(),
                          C.WithBasis(),
                          C,
                          # QQ is not in Modules(QQ)!
                          CommutativeAdditiveSemigroups()]:
                    if codomain in D and domain in D:
                        category = D
                        break
                if category is None:
                    raise ValueError("codomain=(%s) should at least be a commutative additive semigroup")

        H = Hom(domain, codomain, category=category)
        Morphism.__init__(self, H)

        # If needed, this creates the element class from the concrete
        # class (ModuleMorphismByLinearity, DiagonalModuleMorphism,
        # ...) and the abstract classes from the categories.
        #
        # The category infrastructure handles this automatically for
        # parents with a single element class. But for now we still
        # need to do it by hand here, since H may have many different
        # element classes
        if not issubclass(self.__class__, H._abstract_element_class):
            self.__class__ = H.__make_element_class__(self.__class__)


class ModuleMorphismFromFunction(ModuleMorphism, SetMorphism):
    """
    A class for module morphisms implemented by a plain function.

    INPUT:

    - ``domain``, ``codomain``, ``category`` -- as for :class:`ModuleMorphism`

    - ``function`` -- any function or callable from domain to codomain

    .. SEEALSO::

        - :meth:`ModulesWithBasis.ParentMethods.module_morphism` for
          usage information and examples;
        - :mod:`sage.modules.with_basis.morphism` for a technical
          overview of the classes for module morphisms;
        - :class:`ModuleMorphismFromFunction` and
          :class:`TriangularModuleMorphism`.
    """

    def __init__(self, domain, function, codomain=None, category=None):
        """
        TESTS::

            sage: X = CombinatorialFreeModule(ZZ, [1,2,3]); X.rename('X'); x = X.basis()
            sage: from sage.modules.with_basis.morphism import ModuleMorphismFromFunction
            sage: def f(x): return 3*x
            sage: import __main__; __main__.f = f  # Fake f being defined in a python module
            sage: phi = ModuleMorphismFromFunction(X, f, codomain=X)
            sage: phi(x[1])
            3*B[1]
            sage: TestSuite(phi).run()
        """
        # Caveat: This calls Morphism.__init__ twice ...
        ModuleMorphism.__init__(self, domain, codomain, category=category)
        SetMorphism.__init__(self, self.parent(), function)


class ModuleMorphismByLinearity(ModuleMorphism):
    """
    A class for module morphisms obtained by extending a function by linearity.

    INPUT:

    - ``domain``, ``codomain``, ``category`` -- as for :class:`ModuleMorphism`
    - ``on_basis`` -- a function which accepts indices of the basis of
      ``domain`` as ``position``-th argument
    - ``codomain`` -- a parent in ``Modules(...)``
        (default: ``on_basis.codomain()``)
    - ``position`` -- nonnegative integer (default: 0)
    - ``zero`` -- the zero of the codomain (defaults: ``codomain.zero()``)

    .. SEEALSO::

        - :meth:`ModulesWithBasis.ParentMethods.module_morphism` for
          usage information and examples;
        - :mod:`sage.modules.with_basis.morphism` for a technical
          overview of the classes for module morphisms;
        - :class:`ModuleMorphismFromFunction` and
          :class:`TriangularModuleMorphism`.

    .. NOTE::

        ``on_basis`` may alternatively be provided in derived classes
        by passing ``None`` as argument, and implementing or setting
        the attribute ``_on_basis``
    """
    def __init__(self, domain, on_basis=None, codomain=None, category=None,
                 position=0, zero=None):
        """
        TESTS::

            sage: X = CombinatorialFreeModule(ZZ, [-2, -1, 1, 2])
            sage: Y = CombinatorialFreeModule(ZZ, [1, 2])
            sage: from sage.modules.with_basis.morphism import ModuleMorphismByLinearity
            sage: phi = ModuleMorphismByLinearity(X, on_basis=Y.monomial * abs)

            sage: TestSuite(phi).run()
        """
        # Might want to assert that domain is a module with basis
        base_ring = domain.base_ring()

        if codomain is None and hasattr(on_basis, 'codomain'):
            codomain = on_basis.codomain()
        if zero is None:
            zero = codomain.zero()
        self._zero = zero
        self._position = position
        if on_basis is not None:
            self._on_basis = on_basis

        self._is_module_with_basis_over_same_base_ring = \
            codomain in ModulesWithBasis(base_ring) and zero == codomain.zero()

        ModuleMorphism.__init__(self, domain, codomain,
                                category=category,
                                affine=(zero != codomain.zero()))

    def _richcmp_(self, other, op):
        r"""
        Return whether this morphism and ``other`` satisfy ``op``.

        TESTS::

            sage: X = CombinatorialFreeModule(ZZ, [-2, -1, 1, 2])
            sage: Y = CombinatorialFreeModule(ZZ, [1, 2])
            sage: f  = X.module_morphism(on_basis=Y.monomial * abs)
            sage: g  = X.module_morphism(on_basis=Y.monomial * abs)
            sage: h1 = X.module_morphism(on_basis=X.monomial * abs)
            sage: h2 = X.module_morphism(on_basis=X.monomial * factorial)
            sage: h3 = X.module_morphism(on_basis=Y.monomial * abs, category=Modules(ZZ))
            sage: f == g, f == h1, f == h2, f == h3, f == 1, 1 == f
            (True, False, False, False, False, False)
        """
        if op == op_EQ:
            return (self.__class__ is other.__class__
                    and self._zero == other._zero
                    and self._on_basis == other._on_basis
                    and self._position == other._position
                    and self._is_module_with_basis_over_same_base_ring == other._is_module_with_basis_over_same_base_ring)
        if op == op_NE:
            return not (self == other)
        return NotImplemented

    def on_basis(self):
        """
        Return the action of this morphism on basis elements, as per
        :meth:`ModulesWithBasis.Homsets.ElementMethods.on_basis`.

        OUTPUT:

        - a function from the indices of the basis of the domain to the
          codomain

        EXAMPLES::

            sage: X = CombinatorialFreeModule(ZZ, [-2, -1, 1, 2])
            sage: Y = CombinatorialFreeModule(ZZ, [1, 2])
            sage: phi_on_basis = Y.monomial * abs
            sage: phi = sage.modules.with_basis.morphism.ModuleMorphismByLinearity(X, on_basis = phi_on_basis, codomain=Y)
            sage: x = X.basis()
            sage: phi.on_basis()(-2)
            B[2]
            sage: phi.on_basis() == phi_on_basis
            True
        """
        return self._on_basis

    def __call__(self, *args):
        r"""
        Apply this morphism to ``*args``.

        EXAMPLES::

            sage: X = CombinatorialFreeModule(ZZ, [-2, -1, 1, 2])
            sage: Y = CombinatorialFreeModule(ZZ, [1, 2])
            sage: def phi_on_basis(i): return Y.monomial(abs(i))
            sage: phi = sage.modules.with_basis.morphism.ModuleMorphismByLinearity(X, on_basis = Y.monomial * abs, codomain=Y)
            sage: x = X.basis()
            sage: phi(x[1]), phi(x[-2]), phi(x[1] + 3 * x[-2])
            (B[1], B[2], B[1] + 3*B[2])

        .. TODO::

            Add more tests for multi-parameter module morphisms.
        """
        before = args[0:self._position]
        after = args[self._position + 1:len(args)]
        x = args[self._position]
        assert x.parent() is self.domain()

        mc = x.monomial_coefficients(copy=False)
        if self._is_module_with_basis_over_same_base_ring:
            return self.codomain().linear_combination(
                (self._on_basis(*(before + (index,) + after)), coeff)
                for (index, coeff) in mc.items())
        else:
            return sum((coeff * self._on_basis(*(before + (index,) + after))
                        for (index, coeff) in mc.items()), self._zero)

    # As per the specs of Map, we should in fact implement _call_.
    # However we currently need to abuse Map.__call__ (which strict
    # type checking) for multi-parameter module morphisms
    # To be cleaned up
    _call_ = __call__


class TriangularModuleMorphism(ModuleMorphism):
    r"""
    An abstract class for triangular module morphisms.

    Let `X` and `Y` be modules over the same base ring, with
    distinguished bases `F` indexed by `I` and `G` indexed by `J`,
    respectively.

    A module morphism `\phi` from `X` to `Y` is *triangular* if its
    representing matrix in the distinguished bases of `X` and `Y` is
    upper triangular (echelon form).

    More precisely, `\phi` is *upper triangular* w.r.t. a total order
    `<` on `J` if, for any `j\in J`, there exists at most one index
    `i\in I` such that the leading support of `\phi(F_i)` is `j` (see
    :meth:`leading_support()`). We denote by `r(j)` this index,
    setting `r(j)` to ``None`` if it does not exist.

    *Lower triangular* morphisms are defined similarly, taking the
    trailing support instead (see :meth:`trailing_support()`).

    A triangular morphism is *unitriangular* if all its pivots
    (i.e. coefficient of `j` in each `\phi(F[r(j)])`) are `1`.

    INPUT:

    - ``domain`` -- a module with basis `X`
    - ``codomain`` -- a module with basis `Y` (default: `X`)
    - ``category`` -- a category, as for :class:`ModuleMorphism`

    - ``triangular`` -- ``'upper'`` or ``'lower'`` (default: ``'upper'``)
    - ``unitriangular`` -- boolean (default: ``False``)
      As a shorthand, one may use ``unitriangular='lower'``
      for ``triangular='lower', unitriangular=True``.

    - ``key`` -- a comparison key on `J`
      (default: the usual comparison of elements of `J`)

    - ``inverse_on_support`` -- a function `J \to I\cup \{None\}`
      implementing `r` (default: the identity function).
      If set to "compute", the values of `r(j)` are precomputed by
      running through the index set `I` of the basis of the
      domain. This of course requires the domain to be finite
      dimensional.

    - ``invertible`` -- boolean or ``None`` (default: ``None``); can
      be set to specify that `\phi` is known to be (or not to be)
      invertible. If the domain and codomain share the same indexing
      set, this is by default automatically set to ``True`` if
      ``inverse_on_support`` is the identity, or in the finite
      dimensional case.

    .. SEEALSO::

        - :meth:`ModulesWithBasis.ParentMethods.module_morphism` for
          usage information and examples;
        - :mod:`sage.modules.with_basis.morphism` for a technical
          overview of the classes for module morphisms;
        - :class:`ModuleMorphismFromFunction` and
          :class:`TriangularModuleMorphism`.

    OUTPUT: a morphism from `X` to `Y`

    .. WARNING::

        This class is meant to be used as a complement for a concrete
        morphism class.  In particular, the :meth:`__init__` method
        focuses on setting up the data structure describing the
        triangularity of the morphism. It purposely does *not* call
        :meth:`ModuleMorphism.__init__` which should be called
        (directly or indirectly) beforehand.

    EXAMPLES:

    We construct and invert an upper unitriangular module morphism between
    two free `\QQ`-modules::

        sage: I = range(1,200)
        sage: X = CombinatorialFreeModule(QQ, I); X.rename('X'); x = X.basis()
        sage: Y = CombinatorialFreeModule(QQ, I); Y.rename('Y'); y = Y.basis()
        sage: ut = Y.sum_of_monomials * divisors   # This * is map composition.
        sage: phi = X.module_morphism(ut, unitriangular='upper', codomain=Y)
        sage: phi(x[2])
        B[1] + B[2]
        sage: phi(x[6])
        B[1] + B[2] + B[3] + B[6]
        sage: phi(x[30])
        B[1] + B[2] + B[3] + B[5] + B[6] + B[10] + B[15] + B[30]
        sage: phi.preimage(y[2])
        -B[1] + B[2]
        sage: phi.preimage(y[6])
        B[1] - B[2] - B[3] + B[6]
        sage: phi.preimage(y[30])
        -B[1] + B[2] + B[3] + B[5] - B[6] - B[10] - B[15] + B[30]
        sage: (phi^-1)(y[30])
        -B[1] + B[2] + B[3] + B[5] - B[6] - B[10] - B[15] + B[30]

    A lower triangular (but not unitriangular) morphism::

        sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X'); x = X.basis()
        sage: def lt(i): return sum(j*x[j] for j in range(i, 4))
        sage: phi = X.module_morphism(lt, triangular='lower', codomain=X)
        sage: phi(x[2])
        2*B[2] + 3*B[3]
        sage: phi.preimage(x[2])
        1/2*B[2] - 1/2*B[3]
        sage: phi(phi.preimage(x[2]))
        B[2]

    Using the ``key`` keyword, we can use triangularity even if
    the map becomes triangular only after a permutation of the basis::

        sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X'); x = X.basis()
        sage: def ut(i): return (x[1] + x[2] if i == 1 else x[2] + (x[3] if i == 3 else 0))
        sage: perm = [0, 2, 1, 3]
        sage: phi = X.module_morphism(ut, triangular='upper', codomain=X,
        ....:                         key=lambda a: perm[a])
        sage: [phi(x[i]) for i in range(1, 4)]
        [B[1] + B[2], B[2], B[2] + B[3]]
        sage: [phi.preimage(x[i]) for i in range(1, 4)]
        [B[1] - B[2], B[2], -B[2] + B[3]]

    The same works in the lower-triangular case::

        sage: def lt(i): return (x[1] + x[2] + x[3] if i == 2 else x[i])
        sage: phi = X.module_morphism(lt, triangular='lower', codomain=X,
        ....:                         key=lambda a: perm[a])
        sage: [phi(x[i]) for i in range(1, 4)]
        [B[1], B[1] + B[2] + B[3], B[3]]
        sage: [phi.preimage(x[i]) for i in range(1, 4)]
        [B[1], -B[1] + B[2] - B[3], B[3]]

    An injective but not surjective morphism cannot be inverted,
    but the ``inverse_on_support`` keyword allows Sage to find a
    partial inverse::

        sage: X = CombinatorialFreeModule(QQ, [1,2,3]); x = X.basis()
        sage: Y = CombinatorialFreeModule(QQ, [1,2,3,4,5]); y = Y.basis()
        sage: ult = lambda i: sum(  y[j] for j in range(i+1,6)  )
        sage: phi = X.module_morphism(ult, unitriangular='lower', codomain=Y,
        ....:        inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
        sage: phi(x[2])
        B[3] + B[4] + B[5]
        sage: phi.preimage(y[3])
        B[2] - B[3]

    The ``inverse_on_support`` keyword can also be used if the
    bases of the domain and the codomain are identical but one of
    them has to be permuted in order to render the morphism
    triangular. For example::

        sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X'); x = X.basis()
        sage: def ut(i):
        ....:     return (x[3] if i == 1 else x[1] if i == 2
        ....:             else x[1] + x[2])
        sage: def perm(i):
        ....:     return (2 if i == 1 else 3 if i == 2 else 1)
        sage: phi = X.module_morphism(ut, triangular='upper', codomain=X,
        ....:                         inverse_on_support=perm)
        sage: [phi(x[i]) for i in range(1, 4)]
        [B[3], B[1], B[1] + B[2]]
        sage: [phi.preimage(x[i]) for i in range(1, 4)]
        [B[2], -B[2] + B[3], B[1]]

    The same works if the permutation induces lower triangularity::

        sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X'); x = X.basis()
        sage: def lt(i):
        ....:     return (x[3] if i == 1 else x[2] if i == 2
        ....:             else x[1] + x[2])
        sage: def perm(i):
        ....:     return 4 - i
        sage: phi = X.module_morphism(lt, triangular='lower', codomain=X,
        ....:                         inverse_on_support=perm)
        sage: [phi(x[i]) for i in range(1, 4)]
        [B[3], B[2], B[1] + B[2]]
        sage: [phi.preimage(x[i]) for i in range(1, 4)]
        [-B[2] + B[3], B[2], B[1]]

    In the finite dimensional case, one can ask Sage to recover
    ``inverse_on_support`` by a precomputation::

        sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); x = X.basis()
        sage: Y = CombinatorialFreeModule(QQ, [1, 2, 3, 4]); y = Y.basis()
        sage: ut = lambda i: sum(  y[j] for j in range(1,i+2) )
        sage: phi = X.module_morphism(ut, triangular='upper', codomain=Y,
        ....:                         inverse_on_support='compute')
        sage: tx = "{} {} {}"
        sage: for j in Y.basis().keys():
        ....:     i = phi._inverse_on_support(j)
        ....:     print(tx.format(j, i, phi(x[i]) if i is not None else None))
        1 None None
        2 1 B[1] + B[2]
        3 2 B[1] + B[2] + B[3]
        4 3 B[1] + B[2] + B[3] + B[4]

    The ``inverse_on_basis`` and ``key`` keywords can be combined::

        sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X')
        sage: x = X.basis()
        sage: def ut(i):
        ....:     return (2*x[2] + 3*x[3] if i == 1
        ....:             else x[1] + x[2] + x[3] if i == 2
        ....:             else 4*x[2])
        sage: def perm(i):
        ....:     return (2 if i == 1 else 3 if i == 2 else 1)
        sage: perverse_key = lambda a: (a - 2) % 3
        sage: phi = X.module_morphism(ut, triangular='upper', codomain=X,
        ....:                         inverse_on_support=perm, key=perverse_key)
        sage: [phi(x[i]) for i in range(1, 4)]
        [2*B[2] + 3*B[3], B[1] + B[2] + B[3], 4*B[2]]
        sage: [phi.preimage(x[i]) for i in range(1, 4)]
        [-1/3*B[1] + B[2] - 1/12*B[3], 1/4*B[3], 1/3*B[1] - 1/6*B[3]]
    """
    def __init__(self, triangular='upper', unitriangular=False,
                 key=None, inverse=None, inverse_on_support=identity, invertible=None):
        """
        TESTS::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X'); x = X.basis()
            sage: def lt(i): return sum(j*x[j] for j in range(i, 4))
            sage: import __main__; __main__.lt = lt  # Fake lt being defined in a python module
            sage: phi = X.module_morphism(lt, triangular='lower', codomain=X)
            sage: phi.__class__
            <class 'sage.modules.with_basis.morphism.TriangularModuleMorphismByLinearity_with_category'>
            sage: phi._invertible
            True
            sage: TestSuite(phi).run()

        With the option ``compute``::

            sage: phi = X.module_morphism(lt, triangular='lower', codomain=X,
            ....:                         inverse_on_support='compute')
            sage: TestSuite(phi).run(skip=["_test_pickling"])

        Pickling works in Python3 (:issue:`17957`)::

            sage: phi = X.module_morphism(lt, triangular='lower', codomain=X,
            ....:                         inverse_on_support='compute')
            sage: loads(dumps(phi))
            Generic endomorphism of X
            sage: phi._inverse_on_support
            <built-in method get of dict object at ...>
            sage: ldp = loads(dumps(phi._inverse_on_support))
            sage: [ldp(i) == phi._inverse_on_support(i) for i in range(1, 4)]
            [True, True, True]
        """
        if key is not None:
            self._key_kwds = {"key": key}
        else:
            self._key_kwds = {}

        if triangular == "upper":
            self._dominant_item = attrcall("leading_item", **self._key_kwds)
        else:
            self._dominant_item = attrcall("trailing_item", **self._key_kwds)
        # We store those two just be able to pass them down to the inverse function
        self._triangular = triangular

        domain = self.domain()
        codomain = self.codomain()
        on_basis = self.on_basis()
        self._unitriangular = unitriangular
        self._inverse = inverse

        if inverse_on_support == "compute":
            inverse_on_support = {self._dominant_item(on_basis(i))[0]: i
                                  for i in self.domain().basis().keys()
                                  }.get

        self._inverse_on_support = inverse_on_support

        if invertible is None and (domain.basis().keys() == codomain.basis().keys()) and \
           (self._inverse_on_support == identity or domain in Modules.FiniteDimensional):
            invertible = True
        self._invertible = invertible

    def _richcmp_(self, other, op):
        r"""
        Return whether this morphism and ``other`` satisfy ``op``.

        TESTS::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X'); x = X.basis()
            sage: def ut(i): return (x[1] + x[2] if i == 1 else x[2] + (x[3] if i == 3 else 0))
            sage: perm = [0, 2, 1, 3]
            sage: our_key = lambda a: perm[a]
            sage: phi = X.module_morphism(ut, triangular='upper', codomain=X, key=our_key)
            sage: def ut2(i): return (x[1] + 7*x[2] if i == 1 else x[2] + (x[3] if i == 3 else 0))
            sage: phi2 = X.module_morphism(ut2, triangular='upper', codomain=X, key=our_key)
            sage: def lt(i): return (x[1] + x[2] + x[3] if i == 2 else x[i])
            sage: psi = X.module_morphism(lt, triangular='lower', codomain=X, key=our_key)
            sage: phi == phi
            True
            sage: phi == phi2
            False
            sage: phi == psi
            False
        """
        if op == op_EQ:
            return (self.__class__ is other.__class__
                    and self._triangular == other._triangular
                    and self._unitriangular == other._unitriangular
                    and self._inverse_on_support == other._inverse_on_support
                    and self._invertible == other._invertible
                    and self._dominant_item == other._dominant_item)
        if op == op_NE:
            return not (self == other)
        return NotImplemented

    def _test_triangular(self, **options):
        """
        Test that ``self`` is actually triangular.

        See also: :class:`sage.misc.sage_unittest.TestSuite`.

        EXAMPLES::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1, 2, 3]); y = Y.basis()
            sage: lt = lambda i: sum(  y[j] for j in range(i,4)  )
            sage: phi = X.module_morphism(lt, triangular='lower', codomain=Y)
            sage: phi._test_triangular()

            sage: lt = lambda i: sum(  y[j] for j in range(i+1,4)  )
            sage: phi = X.module_morphism(lt, triangular='lower', codomain=Y)
            sage: phi._test_triangular()
            Traceback (most recent call last):
            ...
            AssertionError: morphism is not triangular on 1

            sage: X = CombinatorialFreeModule(QQ, [1,2,3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1,2,3,4,5]); y = Y.basis()
            sage: ult = lambda i: sum(  y[j] for j in range(i+1,6)  )
            sage: phi = X.module_morphism(ult, unitriangular='lower', codomain=Y,
            ....:      inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: phi._test_triangular()

            sage: ult = lambda i: sum(  2*y[j] for j in range(i+1,6)  )
            sage: phi = X.module_morphism(ult, unitriangular='lower', codomain=Y,
            ....:      inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: phi._test_triangular()
            Traceback (most recent call last):
            ...
            AssertionError: morphism is not unitriangular on 1
        """
        from sage.misc.lazy_format import LazyFormat
        tester = self._tester(**options)
        on_basis = self.on_basis()
        for x in self.domain().basis().keys().some_elements():
            # is there any better set to use ?
            bs, co = self._dominant_item(on_basis(x))
            if self._unitriangular:
                tester.assertEqual(co, self.domain().base_ring().one(),
                    LazyFormat("morphism is not unitriangular on %s") % x)
            xback = self._inverse_on_support(bs)
            tester.assertEqual(x, xback,
                LazyFormat("morphism is not triangular on %s") % x)

    def __invert__(self):
        """
        Return the triangular morphism which is the inverse of ``self``.

        Raises an error if ``self`` is not invertible.

        TESTS::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1, 2, 3]); y = Y.basis()
            sage: uut = lambda i: sum(  y[j] for j in range(1,i+1)) # uni-upper
            sage: ult = lambda i: sum(  y[j] for j in range(i,4)  ) # uni-lower
            sage: ut =  lambda i: sum(j*y[j] for j in range(1,i+1)) # upper
            sage: lt =  lambda i: sum(j*y[j] for j in range(i,4  )) # lower
            sage: f_uut = X.module_morphism(uut, codomain=Y,
            ....:                           unitriangular='upper')
            sage: f_ult = X.module_morphism(ult, codomain=Y,
            ....:                           unitriangular='lower')
            sage: f_ut  = X.module_morphism(ut, codomain=Y, triangular='upper')
            sage: f_lt  = X.module_morphism(lt, codomain=Y, triangular='lower')
            sage: (~f_uut)(y[2])
            -B[1] + B[2]
            sage: (~f_ult)(y[2])
            B[2] - B[3]
            sage: (~f_ut)(y[2])
            -1/2*B[1] + 1/2*B[2]
            sage: (~f_lt)(y[2])
            1/2*B[2] - 1/2*B[3]
        """
        if self._invertible is True:
            return self.section()
        elif self._invertible is False:
            raise ValueError("Non invertible morphism")
        else:
            raise ValueError("Morphism not known to be invertible; see the invertible option of module_morphism")

    def section(self):
        """
        Return the section (partial inverse) of ``self``.

        This returns a partial triangular morphism which is a section of
        ``self``. The section morphism raises a :exc:`ValueError` if
        asked to apply on an element which is not in the image of ``self``.

        EXAMPLES::

            sage: X = CombinatorialFreeModule(QQ, [1,2,3]); x = X.basis()
            sage: X.rename('X')
            sage: Y = CombinatorialFreeModule(QQ, [1,2,3,4,5]); y = Y.basis()
            sage: ult = lambda i: sum(  y[j] for j in range(i+1,6)  ) # uni-lower
            sage: phi = X.module_morphism(ult, triangular='lower', codomain=Y,
            ....:      inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: ~phi
            Traceback (most recent call last):
            ...
            ValueError: Morphism not known to be invertible;
            see the invertible option of module_morphism
            sage: phiinv = phi.section()
            sage: list(map(phiinv*phi, X.basis().list())) == X.basis().list()
            True
            sage: phiinv(Y.basis()[1])
            Traceback (most recent call last):
            ...
            ValueError: B[1] is not in the image
        """
        if self._inverse is not None:
            return self._inverse
        if self._inverse_on_support == identity:
            retract_dom = None
        else:
            on_basis = self.on_basis()

            def retract_dom(i):
                self._dominant_item(on_basis(i))[0]

        if self._invertible:
            return self.__class__(
                domain=self.codomain(),
                on_basis=self._invert_on_basis,
                codomain=self.domain(), category=self.category_for(),
                unitriangular=self._unitriangular, triangular=self._triangular,
                inverse=self, inverse_on_support=retract_dom,
                invertible=self._invertible, **self._key_kwds)
        else:
            return SetMorphism(Hom(self.codomain(), self.domain(),
                                   SetsWithPartialMaps()),
                               self.preimage)

    # This should be removed and optimized as soon as triangular
    # morphisms not defined by linearity are available
    # (the inverse should not be computed on the basis).
    def _invert_on_basis(self, i):
        r"""
        Return the image, by the inverse of ``self``, of the basis element
        indexed by ``i``.

        TESTS::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1, 2, 3]); y = Y.basis()
            sage: ult = lambda i: sum(  y[j] for j in range(i,4)  ) # uni-lower
            sage: phi = X.module_morphism(ult, triangular='lower', codomain=Y)
            sage: phi._invert_on_basis(2)
            B[2] - B[3]
        """
        return self.preimage(self.codomain().monomial(i))

    def preimage(self, f):
        r"""
        Return the preimage of `f` under ``self``.

        EXAMPLES::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1, 2, 3]); y = Y.basis()
            sage: ult = lambda i: sum(  y[j] for j in range(i,4)  ) # uni-lower
            sage: phi = X.module_morphism(ult, triangular='lower', codomain=Y)
            sage: phi.preimage(y[1] + y[2])
            B[1] - B[3]

        The morphism need not be surjective. In the following example,
        the codomain is of larger dimension than the domain::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1, 2, 3, 4]); y = Y.basis()
            sage: lt = lambda i: sum(  y[j] for j in range(i,5)  )
            sage: phi = X.module_morphism(lt, triangular='lower', codomain=Y)
            sage: phi.preimage(y[1] + y[2])
            B[1] - B[3]

        Here are examples using ``inverse_on_support`` to handle a
        morphism that shifts the leading indices by `1`::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1, 2, 3, 4, 5]); y = Y.basis()
            sage: lt = lambda i: sum(  y[j] for j in range(i+1,6)  ) # lower
            sage: phi = X.module_morphism(lt, triangular='lower', codomain=Y,
            ....:         inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: phi(x[1])
            B[2] + B[3] + B[4] + B[5]
            sage: phi(x[3])
            B[4] + B[5]
            sage: phi.preimage(y[2] + y[3])
            B[1] - B[3]
            sage: phi(phi.preimage(y[2] + y[3])) == y[2] + y[3]
            True
            sage: el = x[1] + 3*x[2] + 2*x[3]
            sage: phi.preimage(phi(el)) == el
            True

            sage: phi.preimage(y[1])
            Traceback (most recent call last):
            ...
            ValueError: B[1] is not in the image
            sage: phi.preimage(y[4])
            Traceback (most recent call last):
            ...
            ValueError: B[4] is not in the image

        Over a base ring like `\ZZ`, the morphism need not be
        surjective even when the dimensions match::

            sage: X = CombinatorialFreeModule(ZZ, [1, 2, 3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(ZZ, [1, 2, 3]); y = Y.basis()
            sage: lt = lambda i: sum( 2* y[j] for j in range(i,4)  ) # lower
            sage: phi = X.module_morphism(lt, triangular='lower', codomain=Y)
            sage: phi.preimage(2*y[1] + 2*y[2])
            B[1] - B[3]

        The error message in case of failure could be more specific though::

            sage: phi.preimage(y[1] + y[2])
            Traceback (most recent call last):
              ...
            TypeError: no conversion of this rational to integer
        """
        F = self.domain()
        G = self.codomain()
        on_basis = self.on_basis()
        if f not in G:
            raise ValueError("f(={}) must be in the codomain of the morphism to have a preimage under the latter".format(f))

        remainder = f

        out = F.zero()
        while not remainder.is_zero():
            (j, c) = self._dominant_item(remainder)

            j_preimage = self._inverse_on_support(j)
            if j_preimage is None:
                raise ValueError("{} is not in the image".format(f))
            s = on_basis(j_preimage)
            if not j == self._dominant_item(s)[0]:
                raise ValueError("The morphism (={}) is not triangular at {}, and therefore a preimage cannot be computed".format(f, s))

            if not self._unitriangular:
                # What's the appropriate way to request an exact
                # division within the base ring and get an error if
                # this is not possible?
                c = c.parent()(c / s[j])

            # Before this was ``remainder -= s._lmul_(c)`` for speed, but
            #   not every module implements scalar multiplication this way.
            remainder -= s * c
            out += F.term(j_preimage, c)

        return out

    def coreduced(self, y):
        r"""
        Return `y` reduced w.r.t. the image of ``self``.

        INPUT:

        - ``self`` -- a triangular morphism over a field, or a
          unitriangular morphism over a ring
        - ``y`` -- an element of the codomain of ``self``

        Suppose that ``self`` is a morphism from `X` to `Y`. Then, for
        any `y \in Y`, the call ``self.coreduced(y)`` returns a
        normal form for `y` in the quotient `Y / I` where `I` is the
        image of ``self``.

        EXAMPLES::

            sage: X = CombinatorialFreeModule(QQ, [1,2,3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1,2,3,4,5]); y = Y.basis()
            sage: ult = lambda i: sum(  y[j] for j in range(i+1,6)  )
            sage: phi = X.module_morphism(ult, unitriangular='lower', codomain=Y,
            ....:        inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: [phi(v) for v in X.basis()]
            [B[2] + B[3] + B[4] + B[5],
                    B[3] + B[4] + B[5],
                           B[4] + B[5]]
            sage: [phi.coreduced(y[1]-2*y[4])]
            [B[1] + 2*B[5]]
            sage: [phi.coreduced(v) for v in y]
            [B[1], 0, 0, -B[5], B[5]]

        Now with a non unitriangular morphism::

            sage: lt = lambda i: sum( j*y[j] for j in range(i+1,6)  )
            sage: phi = X.module_morphism(lt, triangular='lower', codomain=Y,
            ....:       inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: [phi(v) for v in X.basis()]
            [2*B[2] + 3*B[3] + 4*B[4] + 5*B[5],
                      3*B[3] + 4*B[4] + 5*B[5],
                               4*B[4] + 5*B[5]]
            sage: [phi.coreduced(y[1]-2*y[4])]
            [B[1] + 5/2*B[5]]
            sage: [phi.coreduced(v) for v in y]
            [B[1], 0, 0, -5/4*B[5], B[5]]

        For general rings, this method is only implemented for
        unitriangular morphisms::

            sage: X = CombinatorialFreeModule(ZZ, [1,2,3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(ZZ, [1,2,3,4,5]); y = Y.basis()
            sage: phi = X.module_morphism(ult, unitriangular='lower', codomain=Y,
            ....:       inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: [phi.coreduced(y[1]-2*y[4])]
            [B[1] + 2*B[5]]
            sage: [phi.coreduced(v) for v in y]
            [B[1], 0, 0, -B[5], B[5]]

            sage: phi = X.module_morphism(lt, triangular='lower', codomain=Y,
            ....:       inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: [phi.coreduced(v) for v in y]
            Traceback (most recent call last):
            ...
            NotImplementedError: coreduce for a triangular but not unitriangular morphism over a ring

        .. NOTE:: Before :issue:`8678` this method used to be called co_reduced.
        """
        G = self.codomain()
        if G.base_ring() not in Fields() and not self._unitriangular:
            raise NotImplementedError("coreduce for a triangular but not unitriangular morphism over a ring")
        on_basis = self.on_basis()
        assert y in G

        result = G.zero()
        remainder = y

        while not remainder.is_zero():
            (j, c) = self._dominant_item(remainder)
            j_preimage = self._inverse_on_support(j)
            if j_preimage is None:
                dom_term = G.term(j, c)
                remainder -= dom_term
                result += dom_term
            else:
                s = on_basis(j_preimage)
                assert j == self._dominant_item(s)[0]
                if not self._unitriangular:
                    c = c / s[j]  # the base ring is a field
                # Before this was ``remainder -= s._lmul_(c)`` for speed, but
                #   not every module implements scalar multiplication this way.
                remainder -= s * c
        return result

    def cokernel_basis_indices(self):
        r"""
        Return the indices of the natural monomial basis of the cokernel of ``self``.

        INPUT:

        - ``self`` -- a triangular morphism over a field or a
          unitriangular morphism over a ring, with a finite
          dimensional codomain.

        OUTPUT:

        A list `E` of indices of the basis `(B_e)_e` of the codomain
        of ``self`` so that `(B_e)_{e\in E}` forms a basis of a
        supplementary of the image set of ``self``.

        Thinking of this triangular morphism as a row echelon matrix,
        this returns the complementary of the characteristic
        columns. Namely `E` is the set of indices which do not appear
        as leading support of some element of the image set of
        ``self``.

        EXAMPLES::

            sage: X = CombinatorialFreeModule(ZZ, [1,2,3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(ZZ, [1,2,3,4,5]); y = Y.basis()
            sage: uut = lambda i: sum(  y[j] for j in range(i+1,6)  ) # uni-upper
            sage: phi = X.module_morphism(uut, unitriangular='upper', codomain=Y,
            ....:       inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: phi.cokernel_basis_indices()
            [1, 5]

            sage: phi = X.module_morphism(uut, triangular='upper', codomain=Y,
            ....:       inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: phi.cokernel_basis_indices()
            Traceback (most recent call last):
            ...
            NotImplementedError: cokernel_basis_indices for a triangular but not unitriangular morphism over a ring

            sage: Y = CombinatorialFreeModule(ZZ, NN); y = Y.basis()
            sage: phi = X.module_morphism(uut, unitriangular='upper', codomain=Y,
            ....:       inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: phi.cokernel_basis_indices()
            Traceback (most recent call last):
            ...
            NotImplementedError: cokernel_basis_indices implemented only for morphisms with a finite dimensional codomain
        """
        R = self.domain().base_ring()
        if R not in Fields() and not self._unitriangular:
            raise NotImplementedError("cokernel_basis_indices for a triangular but not unitriangular morphism over a ring")
        if self.codomain() not in Modules(R).FiniteDimensional():
            raise NotImplementedError("cokernel_basis_indices implemented only for morphisms with a finite dimensional codomain")
        return [i for i in self.codomain().basis().keys() if self._inverse_on_support(i) is None]

    def cokernel_projection(self, category=None):
        """
        Return a projection on the co-kernel of ``self``.

        INPUT:

        - ``category`` -- the category of the result

        EXAMPLES::

            sage: X = CombinatorialFreeModule(QQ, [1,2,3]); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1,2,3,4,5]); y = Y.basis()
            sage: lt = lambda i: sum(  y[j] for j in range(i+1,6)  ) # lower
            sage: phi = X.module_morphism(lt, triangular='lower', codomain=Y,
            ....:      inverse_on_support=lambda i: i-1 if i in [2,3,4] else None)
            sage: phipro = phi.cokernel_projection()
            sage: phipro(y[1] + y[2])
            B[1]
            sage: all(phipro(phi(x)).is_zero() for x in X.basis())
            True
            sage: phipro(y[1])
            B[1]
            sage: phipro(y[4])
            -B[5]
            sage: phipro(y[5])
            B[5]
        """
        codomain = self.codomain()
        category = ModulesWithBasis(codomain.base_ring()).or_subcategory(category)
        return codomain.module_morphism(function=self.coreduced,
                                        codomain=codomain, category=category)


class TriangularModuleMorphismByLinearity(ModuleMorphismByLinearity, TriangularModuleMorphism):
    r"""
    A concrete class for triangular module morphisms obtained by extending a function by linearity.

    .. SEEALSO::

        - :meth:`ModulesWithBasis.ParentMethods.module_morphism` for
          usage information and examples;
        - :mod:`sage.modules.with_basis.morphism` for a technical
          overview of the classes for module morphisms;
        - :class:`ModuleMorphismByLinearity` and
          :class:`TriangularModuleMorphism`.
    """
    def __init__(self, domain, on_basis, codomain=None, category=None, **keywords):
        r"""
        TESTS::

            sage: X = CombinatorialFreeModule(QQ, ZZ)
            sage: from sage.modules.with_basis.morphism import TriangularModuleMorphismByLinearity
            sage: def on_basis(i): return X.sum_of_monomials(range(i-2, i+1))
            sage: import __main__; __main__.on_basis = on_basis  # Fake on_basis being defined in a python module
            sage: phi = TriangularModuleMorphismByLinearity(
            ....:           X, on_basis=on_basis, codomain=X)
            sage: TestSuite(phi).run(skip=["_test_nonzero_equal"])
        """
        ModuleMorphismByLinearity.__init__(self, on_basis=on_basis,
                                           domain=domain, codomain=codomain, category=category)
        TriangularModuleMorphism.__init__(self, **keywords)

    def _richcmp_(self, other, op):
        r"""
        Return whether this morphism and ``other`` satisfy ``op``.

        TESTS::

            sage: X = CombinatorialFreeModule(QQ, ZZ)
            sage: from sage.modules.with_basis.morphism import TriangularModuleMorphismByLinearity
            sage: def on_basis(i): return X.sum_of_monomials(range(i-2, i+1))
            sage: phi = TriangularModuleMorphismByLinearity(
            ....:           X, on_basis=on_basis, codomain=X)
            sage: phi == phi
            True
        """
        if op == op_EQ:
            return (ModuleMorphismByLinearity._richcmp_(self, other, op)
                    and TriangularModuleMorphism._richcmp_(self, other, op))
        if op == op_NE:
            return not (self == other)
        return NotImplemented


class TriangularModuleMorphismFromFunction(ModuleMorphismFromFunction, TriangularModuleMorphism):
    r"""
    A concrete class for triangular module morphisms implemented by a function.

    .. SEEALSO::

        - :meth:`ModulesWithBasis.ParentMethods.module_morphism` for
          usage information and examples;
        - :mod:`sage.modules.with_basis.morphism` for a technical
          overview of the classes for module morphisms;
        - :class:`ModuleMorphismFromFunction` and
          :class:`TriangularModuleMorphism`.
    """
    def __init__(self, domain, function, codomain=None, category=None, **keywords):
        r"""
        TESTS::

            sage: X = CombinatorialFreeModule(QQ, [0,1,2,3,4])
            sage: from sage.modules.with_basis.morphism import TriangularModuleMorphismFromFunction
            sage: def f(x): return x + X.term(0, sum(x.coefficients()))
            sage: import __main__; __main__.f = f  # Fake f being defined in a python module
            sage: phi = TriangularModuleMorphismFromFunction(
            ....:           X, function=f, codomain=X)
            sage: TestSuite(phi).run()
        """
        ModuleMorphismFromFunction.__init__(self, function=function,
                                            domain=domain, codomain=codomain,
                                            category=category)
        TriangularModuleMorphism.__init__(self, **keywords)


class ModuleMorphismFromMatrix(ModuleMorphismByLinearity):
    r"""
    A class for module morphisms built from a matrix in the
    distinguished bases of the domain and codomain.

    .. SEEALSO::

        - :meth:`ModulesWithBasis.ParentMethods.module_morphism`
        - :meth:`ModulesWithBasis.FiniteDimensional.MorphismMethods.matrix`

    INPUT:

    - ``domain``, ``codomain`` -- two finite dimensional modules over
      the same base ring `R` with basis `F` and `G`, respectively

    - ``matrix`` -- a matrix with base ring `R` and dimensions
      matching that of `F` and `G`, respectively

    - ``side`` -- ``'left'`` or ``'right'`` (default: ``'left'``)

        If ``side`` is "left", this morphism is considered as
        acting on the left; i.e. each column of the matrix
        represents the image of an element of the basis of the
        domain.

    - ``category`` -- a category or ``None`` (default: ``None``)

    EXAMPLES::

        sage: X = CombinatorialFreeModule(ZZ, [1,2]); X.rename('X'); x = X.basis()
        sage: Y = CombinatorialFreeModule(ZZ, [3,4]); Y.rename('Y'); y = Y.basis()
        sage: m = matrix([[1,2],[3,5]])
        sage: phi = X.module_morphism(matrix=m, codomain=Y)
        sage: phi.parent()
        Set of Morphisms from X to Y in Category of finite dimensional modules with basis over Integer Ring
        sage: phi.__class__
        <class 'sage.modules.with_basis.morphism.ModuleMorphismFromMatrix_with_category'>
        sage: phi(x[1])
        B[3] + 3*B[4]
        sage: phi(x[2])
        2*B[3] + 5*B[4]

        sage: m = matrix([[1,2],[3,5]])
        sage: phi = X.module_morphism(matrix=m, codomain=Y, side='right',
        ....:                         category=Modules(ZZ).WithBasis())
        sage: phi.parent()
        Set of Morphisms from X to Y
        in Category of modules with basis over Integer Ring
        sage: phi(x[1])
        B[3] + 2*B[4]
        sage: phi(x[2])
        3*B[3] + 5*B[4]

    .. TODO::

        Possibly implement rank, addition, multiplication, matrix,
        etc, from the stored matrix.
    """
    def __init__(self, domain, matrix, codomain=None, category=None, side='left'):
        r"""
        Initialize ``self``.

        TESTS::

            sage: from sage.modules.with_basis.morphism import ModuleMorphismFromMatrix
            sage: X = CombinatorialFreeModule(ZZ, [1,2]); X.rename('X'); x = X.basis()
            sage: Y = CombinatorialFreeModule(ZZ, [3,4]); Y.rename('Y'); y = Y.basis()
            sage: m = matrix([[1,2],[3,5]])
            sage: phi = ModuleMorphismFromMatrix(matrix=m, domain=X, codomain=Y, side='right')
            sage: phi.__class__
            <class 'sage.modules.with_basis.morphism.ModuleMorphismFromMatrix_with_category'>
            sage: phi.matrix(side='right') == m
            True
            sage: TestSuite(phi).run(skip=["_test_pickling"])

        Pickling works (:issue:`17957`) in Python 3::

            sage: phi._on_basis
            <built-in method __getitem__ of dict object at ...>
            sage: loads(dumps(phi)) == phi
            True

        The matrix is stored in the morphism, as if it was for an
        action on the right::

            sage: phi._matrix
            [1 2]
            [3 5]

            sage: phi = ModuleMorphismFromMatrix(matrix=m, side='left',
            ....:                                domain=X, codomain=Y)
            sage: phi._matrix
            [1 3]
            [2 5]
        """
        C = ModulesWithBasis(domain.base_ring()).FiniteDimensional()
        if domain not in C:
            raise ValueError("The domain %s should be finite dimensional" % domain)
        if codomain is None:
            raise ValueError("The codomain %s should be specified")
        if codomain not in C:
            raise ValueError("The codomain %s should be finite dimensional" % codomain)
        if not isinstance(matrix, Matrix):
            raise ValueError("matrix (=%s) should be a matrix" % matrix)
        import sage.combinat.ranker
        indices = tuple(domain.basis().keys())
        rank_domain = sage.combinat.ranker.rank_from_list(indices)
        if side == "left":
            matrix = matrix.transpose()
        if matrix.nrows() != len(indices):
            raise ValueError("The dimension of the matrix (%s) does not match with the dimension of the domain (%s)"
                             % (matrix.nrows(), len(indices)))
        if matrix.ncols() != codomain.dimension():
            raise ValueError("The dimension of the matrix (%s) does not match with the dimension of the codomain (%s)"
                             % (matrix.ncols(), codomain.dimension()))
        self._matrix = matrix
        d = {xt: codomain.from_vector(matrix.row(rank_domain(xt)))
             for xt in domain.basis().keys()}

        ModuleMorphismByLinearity.__init__(self, on_basis=d.__getitem__,
                                           domain=domain, codomain=codomain,
                                           category=category)

    def _richcmp_(self, other, op):
        r"""
        Return whether this morphism and ``other`` satisfy ``op``.

        TESTS::

            sage: from sage.modules.with_basis.morphism import ModuleMorphismFromMatrix
            sage: X = CombinatorialFreeModule(ZZ, [1,2]); X.rename('X'); x = X.basis()
            sage: Y = CombinatorialFreeModule(ZZ, [3,4]); Y.rename('Y'); y = Y.basis()
            sage: m = matrix([[1,2],[3,5]])
            sage: phi = ModuleMorphismFromMatrix(matrix=m, domain=X, codomain=Y, side='right')
            sage: phi2 = ModuleMorphismFromMatrix(matrix=m, domain=X, codomain=Y, side='right')
            sage: phi == phi2
            True
            sage: phi is phi2
            False
            sage: m2 = matrix([[1,2],[4,5]])
            sage: phi2 = ModuleMorphismFromMatrix(matrix=m2, domain=X, codomain=Y, side='right')
            sage: phi == phi2
            False
        """
        if op == op_EQ:
            # We skip the on_basis check since the matrix defines the morphism
            return (self.__class__ is other.__class__
                    and self._zero == other._zero
                    and self._position == other._position
                    and self._is_module_with_basis_over_same_base_ring == other._is_module_with_basis_over_same_base_ring
                    and self._matrix == other._matrix)
        if op == op_NE:
            return not (self == other)
        return NotImplemented


class DiagonalModuleMorphism(ModuleMorphismByLinearity):
    r"""
    A class for diagonal module morphisms.

    See :meth:`ModulesWithBasis.ParentMethods.module_morphism`.

    INPUT:

    - ``domain``, ``codomain`` -- two modules with basis `F` and `G`,
      respectively
    - ``diagonal`` -- a function `d`

    Assumptions:

    - ``domain`` and ``codomain`` have the same base ring `R`,
    - their respective bases `F` and `G` have the same index set `I`,
    - `d` is a function `I \to R`.

    Return the diagonal module morphism from ``domain`` to ``codomain``
    sending `F(i) \mapsto d(i) G(i)` for all `i \in I`.

    By default, ``codomain`` is currently assumed to be ``domain``.
    (Todo: make a consistent choice with ``*ModuleMorphism``.)

    .. TODO::

        - Implement an optimized ``_call_()`` function.
        - Generalize to a mapcoeffs.
        - Generalize to a mapterms.

    EXAMPLES::

        sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X')
        sage: phi = X.module_morphism(diagonal=factorial, codomain=X)
        sage: x = X.basis()
        sage: phi(x[1]), phi(x[2]), phi(x[3])
        (B[1], 2*B[2], 6*B[3])
    """
    def __init__(self, domain, diagonal, codomain=None, category=None):
        r"""
        Initialize ``self``.

        TESTS::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X')
            sage: phi = X.module_morphism(diagonal=factorial, codomain=X)
            sage: phi.__class__
            <class 'sage.modules.with_basis.morphism.DiagonalModuleMorphism_with_category'>
            sage: TestSuite(phi).run()
        """
        if codomain is None:
            raise ValueError("The codomain should be specified")
        if not (domain.basis().keys() == codomain.basis().keys() and
                domain.base_ring() == codomain.base_ring()):
            raise ValueError("The domain and codomain should have the same base ring "
                             "and the same basis indexing")
        from collections.abc import Callable
        if not isinstance(diagonal, Callable):
            raise ValueError("diagonal (=%s) should be a function" % diagonal)
        if category is None:
            category = ModulesWithBasis(domain.base_ring())
        ModuleMorphismByLinearity.__init__(
            self, domain=domain, codomain=codomain, category=category)
        self._diagonal = diagonal

    def _richcmp_(self, other, op):
        r"""
        Return whether this morphism and ``other`` satisfy ``op``.

        TESTS::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X')
            sage: phi = X.module_morphism(diagonal=factorial, codomain=X)
            sage: psi = X.module_morphism(diagonal=factorial, codomain=X)
            sage: phi == psi
            True
            sage: phi is psi
            False
        """
        if op == op_EQ:
            return (self.__class__ is other.__class__
                    and self._diagonal == other._diagonal)
        if op == op_NE:
            return not (self == other)
        return NotImplemented

    def _on_basis(self, i):
        """
        Return the image by ``self`` of the basis element indexed by ``i``.

        TESTS::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X'); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1, 2, 3]); Y.rename('Y'); y = Y.basis()
            sage: phi = X.module_morphism(diagonal=factorial, codomain=X)
            sage: phi._on_basis(3)
            6*B[3]
        """
        return self.codomain().term(i, self._diagonal(i))

    def __invert__(self):
        r"""
        Return the inverse diagonal morphism.

        EXAMPLES::

            sage: X = CombinatorialFreeModule(QQ, [1, 2, 3]); X.rename('X'); x = X.basis()
            sage: Y = CombinatorialFreeModule(QQ, [1, 2, 3]); Y.rename('Y'); y = Y.basis()
            sage: phi = X.module_morphism(diagonal=factorial, codomain=X)
            sage: phi_inv = ~phi
            sage: phi_inv
            Generic endomorphism of Y
            sage: phi_inv(y[3])
            1/6*B[3]

        Caveat: this inverse morphism is only well defined if
        `d(\lambda)` is always invertible in the base ring. This
        condition is *not* tested for, so using an ill defined inverse
        morphism will trigger arithmetic errors.
        """
        return self.codomain().module_morphism(
            diagonal=pointwise_inverse_function(self._diagonal),
            codomain=self.domain(), category=self.category_for())


def pointwise_inverse_function(f):
    r"""
    Return the function `x \mapsto 1 / f(x)`.

    INPUT:

    - ``f`` -- a function

    EXAMPLES::

        sage: from sage.modules.with_basis.morphism import pointwise_inverse_function
        sage: def f(x): return x
        sage: g = pointwise_inverse_function(f)
        sage: g(1), g(2), g(3)
        (1, 1/2, 1/3)

    :func:`pointwise_inverse_function` is an involution::

        sage: f is pointwise_inverse_function(g)
        True

    .. TODO::

        This has nothing to do here!!! Should there be a library for
        pointwise operations on functions somewhere in Sage?
    """
    if hasattr(f, "pointwise_inverse"):
        return f.pointwise_inverse()
    return PointwiseInverseFunction(f)


class PointwiseInverseFunction(SageObject):
    r"""
    A class for pointwise inverse functions.

    The pointwise inverse function of a function `f` is the function
    sending every `x` to `1 / f(x)`.

    EXAMPLES::

        sage: from sage.modules.with_basis.morphism import PointwiseInverseFunction
        sage: f = PointwiseInverseFunction(factorial)
        sage: f(0), f(1), f(2), f(3)
        (1, 1, 1/2, 1/6)
    """
    def __init__(self, f):
        """
        TESTS::

            sage: from sage.modules.with_basis.morphism import PointwiseInverseFunction
            sage: f = PointwiseInverseFunction(factorial)
            sage: f(0), f(1), f(2), f(3)
            (1, 1, 1/2, 1/6)
            sage: TestSuite(f).run()
        """
        self._pointwise_inverse = f

    def __eq__(self, other):
        r"""
        Return whether this function is equal to ``other``.

        TESTS::

            sage: from sage.modules.with_basis.morphism import PointwiseInverseFunction
            sage: f = PointwiseInverseFunction(factorial)
            sage: g = PointwiseInverseFunction(factorial)
            sage: f is g
            False
            sage: f == g
            True
        """
        return (self.__class__ is other.__class__
                and self._pointwise_inverse == other._pointwise_inverse)

    def __ne__(self, other):
        r"""
        Return whether this function is not equal to ``other``.

        TESTS::

            sage: from sage.modules.with_basis.morphism import PointwiseInverseFunction
            sage: f = PointwiseInverseFunction(factorial)
            sage: g = PointwiseInverseFunction(factorial)
            sage: f != g
            False
        """
        return not (self == other)

    def __call__(self, *args):
        """
        TESTS::

            sage: from sage.modules.with_basis.morphism import PointwiseInverseFunction
            sage: g = PointwiseInverseFunction(operator.mul)
            sage: g(5,7)
            1/35
        """
        return ~(self._pointwise_inverse(*args))

    def pointwise_inverse(self):
        """
        TESTS::

            sage: from sage.modules.with_basis.morphism import PointwiseInverseFunction
            sage: g = PointwiseInverseFunction(operator.mul)
            sage: g.pointwise_inverse() is operator.mul
            True
        """
        return self._pointwise_inverse



# sage_setup: distribution = sagemath-flint
r"""
Deprecated module.

Functions were moved in qsieve_sage.pyx.

TESTS::

    sage: from sage.libs.flint.qsieve import qsieve
    sage: qsieve(1000)
    doctest:warning
    ...
    DeprecationWarning:
    Importing qsieve from here is deprecated; please use "from sage.libs.flint.qsieve_sage import qsieve" instead.
    See https://github.com/sagemath/sage/issues/36449 for details.
    [(2, 3), (5, 3)]
"""

from sage.misc.lazy_import import LazyImport

qsieve = LazyImport('sage.libs.flint.qsieve_sage', 'qsieve', deprecation=36449)



# sage_setup: distribution = sagemath-modules
# sage.doctest: needs sage.rings.finite_rings

#############################################################################
#    Copyright (C) 2019 Xavier Caruso <xavier.caruso@normalesup.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 2 of the License, or
#    (at your option) any later version.
#                  http://www.gnu.org/licenses/
#****************************************************************************


from sage.structure.parent cimport Parent
from sage.structure.element cimport Element
from sage.categories.pushout import construction_tower
from sage.categories.map cimport Map, FormalCompositeMap
from sage.categories.morphism import IdentityMorphism
from sage.rings.ring_extension cimport RingExtension_generic
from sage.rings.ring_extension_element cimport RingExtensionElement
from sage.rings.ring_extension_morphism cimport RingExtensionHomomorphism
from sage.rings.ring_extension_morphism cimport RingExtensionBackendIsomorphism
from sage.rings.ring_extension_morphism cimport RingExtensionBackendReverseIsomorphism


# For parents
#############

cpdef backend_parent(R):
    r"""
    Return the backend parent of ``R``.

    INPUT:

    - ``R`` -- a parent

    EXAMPLES:

        sage: from sage.rings.ring_extension_conversion import backend_parent

        sage: K.<a> = GF(5^2).over()  # over GF(5)
        sage: backend_parent(K)
        Finite Field in z2 of size 5^2
        sage: backend_parent(K) is GF(5^2)
        True
    """
    if isinstance(R, RingExtension_generic):
        return (<RingExtension_generic>R)._backend
    else:
        return R

cpdef from_backend_parent(R, RingExtension_generic E):
    r"""
    Try to reconstruct a ring extension (somehow related to ``E``)
    whose backend is ``R``.

    INPUT:

    - ``R`` -- a parent

    - ``E`` -- a ring extension

    EXAMPLES::

        sage: from sage.rings.ring_extension_conversion import from_backend_parent

        sage: K.<a> = GF(5^2).over()   # over GF(5)
        sage: L.<b> = GF(5^4).over(K)

        sage: from_backend_parent(GF(5^2), K)
        Field in a with defining polynomial x^2 + 4*x + 2 over its base
        sage: from_backend_parent(GF(5^2), K) is K
        True

    Bases are recognized::

        sage: from_backend_parent(GF(5^2), L)
        Field in a with defining polynomial x^2 + 4*x + 2 over its base
        sage: from_backend_parent(GF(5^2), L) is K
        True

    And also certain constructions::

        sage: S.<x> = GF(5^2)[]
        sage: T = from_backend_parent(S, L)
        sage: T
        Univariate Polynomial Ring in x over Field in a with defining polynomial x^2 + 4*x + 2 over its base
        sage: T.base_ring() is K
        True
    """
    tower = construction_tower(R)
    bases_tower = [ parent for (_, parent) in tower ]
    for base in E.bases():
        backend = backend_parent(base)
        try:
            s = bases_tower.index(backend)
            ans = base
            for i in range(s, 0, -1):
                functor = tower[i][0]
                ans = functor(ans)
            return ans
        except ValueError:
            pass
    return R


# For elements
##############

cpdef backend_element(x):
    r"""
    Return the backend element of ``x``.

    INPUT:

    - ``x`` -- an element

    EXAMPLES:

        sage: from sage.rings.ring_extension_conversion import backend_element

        sage: K.<a> = GF(5^2).over()   # over GF(5)
        sage: backend_element(a)
        z2
        sage: backend_element(a) in GF(5^2)
        True
    """
    if isinstance(x, RingExtensionElement):
        return (<RingExtensionElement>x)._backend
    else:
        return x

cpdef from_backend_element(x, RingExtension_generic E):
    r"""
    Try to reconstruct an element in a ring extension (somehow
    related to ``E``) whose backend is ``x``.

    INPUT:

    - ``x`` -- an element

    - ``E`` -- a ring extension

    EXAMPLES::

        sage: from sage.rings.ring_extension_conversion import from_backend_element

        sage: K.<a> = GF(5^2).over()   # over GF(5)
        sage: L.<b> = GF(5^4).over(K)
        sage: z2 = GF(5^2).gen()

        sage: from_backend_element(z2, K)
        a
        sage: from_backend_element(z2, K).parent() is K
        True

    Bases are recognized::

        sage: from_backend_element(z2, L)
        a
        sage: from_backend_element(z2, L).parent() is K
        True

    And also certain constructions::

        sage: S.<x> = GF(5^2)[]
        sage: u = from_backend_element(x + z2, L); u
        x + a
        sage: u.parent()
        Univariate Polynomial Ring in x over Field in a with defining polynomial x^2 + 4*x + 2 over its base
        sage: u.base_ring() is K
        True
    """
    parent = from_backend_parent(x.parent(),E)
    if parent is None:
        return x
    else:
        return parent(x)


# For morphisms
###############

cdef _backend_morphism(f):
    r"""
    Return the backend morphism of ``f``.

    INPUT:

    - ``f`` -- a map

    TESTS::

        sage: from sage.rings.ring_extension_conversion import backend_morphism

        sage: Frob = GF(7^3).frobenius_endomorphism()
        sage: backend_morphism(Frob) is Frob   # indirect doctest
        True

        sage: K.<a> = GF(7^3).over()
        sage: f = End(K)(Frob)
        sage: type(f)
        <class 'sage.rings.ring_extension_morphism.RingExtensionHomomorphism'>
        sage: backend_morphism(f) == Frob   # indirect doctest
        True

        sage: L.<b> = GF(7^6).over(K)
        sage: g = f.extend_codomain(L)
        sage: bg = backend_morphism(g); bg
        Composite map:
          From: Finite Field in z3 of size 7^3
          To:   Finite Field in z6 of size 7^6
          Defn:   Frobenius endomorphism z3 |--> z3^7 on Finite Field in z3 of size 7^3
                then
                  Ring morphism:
                  From: Finite Field in z3 of size 7^3
                  To:   Finite Field in z6 of size 7^6
                  Defn: z3 |--> 2*z6^4 + 6*z6^3 + 2*z6^2 + 3*z6 + 2
        sage: backend_morphism(bg) == bg
        True

        sage: iota = End(K).identity()
        sage: type(iota)
        <class 'sage.categories.morphism.IdentityMorphism'>
        sage: backend_morphism(iota)
        Identity endomorphism of Finite Field in z3 of size 7^3
    """
    domain = f.domain()
    if not isinstance(f.domain(), RingExtension_generic) and not isinstance(f.codomain(), RingExtension_generic):
        return f
    elif isinstance(f, RingExtensionHomomorphism):
        return (<RingExtensionHomomorphism>f)._backend
    elif isinstance(f, FormalCompositeMap):
        return _backend_morphism(f.then()) * _backend_morphism(f.first())
    elif isinstance(f, IdentityMorphism):
        ring = backend_parent(domain)
        return ring.Hom(ring).identity()
    elif domain is domain.base_ring():
        ring = f.codomain()
        if isinstance(ring, RingExtension_generic):
            ring = ring._backend
        if ring.has_coerce_map_from(domain):
            return ring.coerce_map_from(domain)
    raise NotImplementedError

cpdef backend_morphism(f, forget='all'):
    r"""
    Return the backend morphism of ``f``.

    INPUT:

    - ``f`` -- a map

    - ``forget`` -- string; either ``'all'`` or ``'domain'`` or ``'codomain'``
      (default: ``'all'``). Whether to switch to the backend for the domain,
      the codomain or both of them.

    EXAMPLES::

        sage: from sage.rings.ring_extension_conversion import backend_morphism

        sage: K.<a> = GF(7^3).over()   # over GF(7)
        sage: f = K.hom([a^7])
        sage: f
        Ring endomorphism of Field in a with defining polynomial x^3 + 6*x^2 + 4 over its base
          Defn: a |--> 5*a + 3*a^2

        sage: backend_morphism(f)
        Ring endomorphism of Finite Field in z3 of size 7^3
          Defn: z3 |--> 3*z3^2 + 5*z3

        sage: backend_morphism(f, forget='domain')
        Ring morphism:
          From: Finite Field in z3 of size 7^3
          To:   Field in a with defining polynomial x^3 + 6*x^2 + 4 over its base
          Defn: z3 |--> 5*a + 3*a^2

        sage: backend_morphism(f, forget='codomain')
        Ring morphism:
          From: Field in a with defining polynomial x^3 + 6*x^2 + 4 over its base
          To:   Finite Field in z3 of size 7^3
          Defn: a |--> 3*z3^2 + 5*z3
    """
    try:
        g = _backend_morphism(f)
        if forget is None and (isinstance(f.domain(), RingExtension_generic) or isinstance(f.codomain(), RingExtension_generic)):
            g = RingExtensionHomomorphism(f.domain().Hom(f.codomain()), g)
        if forget == "domain" and isinstance(f.codomain(), RingExtension_generic):
            g = RingExtensionHomomorphism(g.domain().Hom(f.codomain()), g)
        if forget == "codomain" and isinstance(f.domain(), RingExtension_generic):
            g = RingExtensionHomomorphism(f.domain().Hom(g.codomain()), g)
    except NotImplementedError:
        g = f
        if (forget == "all" or forget == "domain") and isinstance(f.domain(), RingExtension_generic):
            ring = f.domain()._backend
            g = g * RingExtensionBackendIsomorphism(ring.Hom(f.domain()))
        if (forget == "all" or forget == "codomain") and isinstance(f.codomain(), RingExtension_generic):
            ring = f.codomain()._backend
            g = RingExtensionBackendReverseIsomorphism(f.codomain().Hom(ring)) * g
    return g

cpdef from_backend_morphism(f, RingExtension_generic E):
    r"""
    Try to reconstruct a morphism between ring extensions
    (somehow related to ``E``) whose backend is ``f``.

    INPUT:

    - ``x`` -- a morphism

    - ``E`` -- a ring extension

    EXAMPLES::

        sage: from sage.rings.ring_extension_conversion import from_backend_morphism

        sage: K.<a> = GF(5^2).over()  # over GF(5)
        sage: L.<b> = GF(5^6).over(K)

        sage: Frob = GF(5^2).frobenius_endomorphism()
        sage: from_backend_morphism(Frob, K)
        Ring endomorphism of Field in a with defining polynomial x^2 + 4*x + 2 over its base
          Defn: a |--> 1 - a

    Bases are recognized::

        sage: from_backend_morphism(Frob, L)
        Ring endomorphism of Field in a with defining polynomial x^2 + 4*x + 2 over its base
          Defn: a |--> 1 - a
    """
    cdef domain = from_backend_parent(f.domain(), E)
    cdef codomain = from_backend_parent(f.codomain(), E)
    return RingExtensionHomomorphism(domain.Hom(codomain), f)


# Generic
#########

cpdef to_backend(arg):
    r"""
    Return the backend of ``arg``.

    EXAMPLES::

        sage: from sage.rings.ring_extension_conversion import to_backend

    This function accepts parents::

        sage: K.<a> = GF(5^2).over()   # over GF(5)
        sage: to_backend(K)
        Finite Field in z2 of size 5^2

    elements::

        sage: to_backend(a)
        z2

    morphisms::

        sage: f = K.hom([a^5])
        sage: to_backend(f)
        Ring endomorphism of Finite Field in z2 of size 5^2
          Defn: z2 |--> 4*z2 + 1

    list/tuple of them::

        sage: to_backend(([K, a], f))
        ([Finite Field in z2 of size 5^2, z2],
         Ring endomorphism of Finite Field in z2 of size 5^2
           Defn: z2 |--> 4*z2 + 1)

    and dictionaries::

        sage: to_backend({a: K})
        {z2: Finite Field in z2 of size 5^2}

    .. SEEALSO::

        :meth:`to_backend_parent`, :meth:`to_backend_element`, :meth:`to_backend_morphism`
    """
    if isinstance(arg, list):
        return [ to_backend(x) for x in arg ]
    elif isinstance(arg, tuple):
        return tuple([ to_backend(x) for x in arg ])
    elif isinstance(arg, dict):
        return { to_backend(key): to_backend(value) for (key, value) in arg.items() }
    elif isinstance(arg, RingExtension_generic):
        return (<RingExtension_generic>arg)._backend
    elif isinstance(arg, Map):
        return backend_morphism(arg)
    elif isinstance(arg, RingExtensionElement):
        return (<RingExtensionElement>arg)._backend
    return arg

cpdef from_backend(arg, E):
    r"""
    Try to reconstruct something (somehow related to ``E``)
    whose backend is ``arg``.

    INPUT:

    - ``arg`` -- any argument

    - ``E`` -- a ring extension

    EXAMPLES::

        sage: from sage.rings.ring_extension_conversion import from_backend

    This function accepts parents::

        sage: K.<a> = GF(5^2).over()   # over GF(5)
        sage: from_backend(GF(5^2), K)
        Field in a with defining polynomial x^2 + 4*x + 2 over its base

    elements::

        sage: z2 = GF(5^2).gen()
        sage: from_backend(z2, K)
        a

    morphisms::

        sage: f = GF(5^2).frobenius_endomorphism()
        sage: from_backend(f, K)
        Ring endomorphism of Field in a with defining polynomial x^2 + 4*x + 2 over its base
          Defn: a |--> 1 - a

    list/tuple of them::

        sage: from_backend(([K, a], f), K)
        ([Field in a with defining polynomial x^2 + 4*x + 2 over its base, a],
         Ring endomorphism of Field in a with defining polynomial x^2 + 4*x + 2 over its base
           Defn: a |--> 1 - a)

    and dictionaries::

        sage: from_backend({a: K}, K)
        {a: Field in a with defining polynomial x^2 + 4*x + 2 over its base}

    .. SEEALSO::

        :meth:`from_backend_parent`, :meth:`from_backend_element`, :meth:`from_backend_morphism`
    """
    ans = None
    if isinstance(arg, list):
        ans = [ from_backend(x,E) for x in arg ]
    elif isinstance(arg, tuple):
        ans = tuple([ from_backend(x,E) for x in arg ])
    elif isinstance(arg, dict):
        ans = { from_backend(key,E): from_backend(value,E) for (key, value) in arg.items() }
    elif isinstance(arg, Parent):
        ans = from_backend_parent(arg,E)
    elif isinstance(arg, Map):
        ans = from_backend_morphism(arg,E)
    elif isinstance(arg, Element):
        ans = from_backend_element(arg,E)
    if ans is None:
        return arg
    else:
        return ans



# sage_setup: distribution = sagemath-flint
# sage.doctest: # needs sage.symbolic
"""
Real intervals with a fixed absolute precision
"""

from sage.ext.stdsage cimport PY_NEW

from sage.libs.gmp.mpz cimport *

from sage.structure.factory import UniqueFactory
from sage.structure.element cimport Element, FieldElement
from sage.rings.ring cimport Field
from sage.rings.integer cimport Integer
import sage.rings.abc

from sage.structure.parent cimport Parent
from sage.structure.element cimport parent as parent_of

from sage.rings.real_mpfr import RR_min_prec
from sage.rings.real_mpfi import RealIntervalField, RealIntervalFieldElement
from sage.rings.rational_field import QQ

cdef Integer zero = Integer(0)
cdef Integer one = Integer(1)

cpdef inline Integer shift_floor(Integer x, long shift):
    r"""
    Return `x / 2^s` where `s` is the value of ``shift``, rounded towards
    `-\infty`. For internal use.

    EXAMPLES::

        sage: from sage.rings.real_interval_absolute import shift_floor
        sage: shift_floor(15, 2)
        3
        sage: shift_floor(-15, 2)
        -4
    """
    cdef Integer z = PY_NEW(Integer)
    mpz_fdiv_q_2exp(z.value, x.value, shift)
    return z

cpdef inline Integer shift_ceil(Integer x, long shift):
    r"""
    Return `x / 2^s` where `s` is the value of ``shift``, rounded towards
    `+\infty`. For internal use.

    EXAMPLES::

        sage: from sage.rings.real_interval_absolute import shift_ceil
        sage: shift_ceil(15, 2)
        4
        sage: shift_ceil(-15, 2)
        -3
        sage: shift_ceil(32, 2)
        8
        sage: shift_ceil(-32, 2)
        -8
    """
    cdef Integer z = PY_NEW(Integer)
    mpz_cdiv_q_2exp(z.value, x.value, shift)
    return z


class Factory(UniqueFactory):
    def create_key(self, prec):
        """
        The only piece of data is the precision.

        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: RealIntervalAbsoluteField.create_key(1000)
            1000
        """
        return prec

    def create_object(self, version, prec):
        """
        Ensures uniqueness.

        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: RealIntervalAbsoluteField(23) is RealIntervalAbsoluteField(23) # indirect doctest
            True
        """
        return RealIntervalAbsoluteField_class(prec)


RealIntervalAbsoluteField = Factory('sage.rings.real_interval_absolute.RealIntervalAbsoluteField')
RealIntervalAbsoluteField.__doc__ = RealIntervalAbsoluteField_class.__doc__


cdef class RealIntervalAbsoluteField_class(Field):
    """
    This field is similar to the :class:`RealIntervalField` except instead of
    truncating everything to a fixed relative precision, it maintains a
    fixed absolute precision.

    Note that unlike the standard real interval field, elements in this
    field can have different size and experience coefficient blowup. On
    the other hand, it avoids precision loss on addition and subtraction.
    This is useful for, e.g., series computations for special functions.

    EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10); R
            Real Interval Field with absolute precision 2^-10
            sage: R(3/10)
            0.300?
            sage: R(1000003/10)
            100000.300?
            sage: R(1e100) + R(1) - R(1e100)
            1
    """

    cdef long _absprec

    def __init__(self, absprec):
        """
        Initialize ``self``.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: RealIntervalAbsoluteField(100)
            Real Interval Field with absolute precision 2^-100
            sage: RealIntervalAbsoluteField(-100)
            Traceback (most recent call last):
              File "<ipython console>", line 1, in <module>
              File "real_interval_absolute.pyx", line 81, in sage.rings.real_interval_absolute.RealIntervalAbsoluteField.__init__ (sage/rings/real_interval_absolute.c:2463)
            ValueError: Absolute precision must be positive.
        """
        if absprec < 0:
            raise ValueError("Absolute precision must be positive.")
        Field.__init__(self, self)
        self._absprec = absprec

    def __reduce__(self):
        """
        Used for pickling.

        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: loads(dumps(R))
            Real Interval Field with absolute precision 2^-100
        """
        return RealIntervalAbsoluteField, (self._absprec,)

    def _element_constructor_(self, x):
        """
        Construct an element with ``self`` as the parent.

        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: R(1/2) # indirect doctest
            0.50000000000000000000000000000000?
        """
        return RealIntervalAbsoluteElement(self, x)

    cpdef _coerce_map_from_(self, R):
        """
        Anything that coerces into the reals coerces into this ring.

        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: R.has_coerce_map_from(RR) # indirect doctest
            True
            sage: R.has_coerce_map_from(QQ)
            True
            sage: R.has_coerce_map_from(Qp(5))                                          # needs sage.rings.padics
            False

            sage: R(1/2) + 100
            100.5000000000000000000000000000000?
            sage: R(1/2) + 1.75
            2.2500000000000000000000000000000?

            sage: R10 = RealIntervalAbsoluteField(10)
            sage: R10(1/4) + R(1/4)
            0.50000?
        """
        if isinstance(R, RealIntervalAbsoluteField_class):
            return self._absprec < (<RealIntervalAbsoluteField_class>R)._absprec
        elif isinstance(R, sage.rings.abc.RealIntervalField):
            return True
        else:
            return RR_min_prec.has_coerce_map_from(R)

    def _repr_(self):
        """
        Return the string representation of ``self``.

        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: print(R)
            Real Interval Field with absolute precision 2^-100
            sage: R._repr_()
            'Real Interval Field with absolute precision 2^-100'
        """
        return "Real Interval Field with absolute precision 2^-%s" % self._absprec

    def absprec(self):
        """
        Return the absolute precision of ``self``.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: R.absprec()
            100
            sage: RealIntervalAbsoluteField(5).absprec()
            5
        """
        return self._absprec


cdef inline shift_left(value, shift):
    """
    Utility function for operands that don't support the ``<<`` operator.
    """
    try:
        return value << shift
    except TypeError:
        if isinstance(value, (str, list, tuple)):
            # Better than the OverflowError we would get from trying to multiply.
            raise
        else:
            return value * (one << shift)

cdef class RealIntervalAbsoluteElement(FieldElement):

    # This could be optimized by letting these be raw mpz_t.
    cdef Integer _mantissa  # left endpoint
    cdef Integer _diameter

    def __init__(self, RealIntervalAbsoluteField_class parent, value):
        """
        Create a :class:`RealIntervalAbsoluteElement`.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(50)
            sage: R(1)
            1
            sage: R(1/3)
            0.333333333333334?
            sage: R(1.3)
            1.300000000000000?
            sage: R(pi)
            3.141592653589794?
            sage: R((11, 12))
            12.?
            sage: R((11, 11.00001))
            11.00001?

            sage: R100 = RealIntervalAbsoluteField(100)
            sage: R(R100((5,6)))
            6.?
            sage: R100(R((5,6)))
            6.?
            sage: RIF(CIF(NaN))
            [.. NaN ..]
        """
        Element.__init__(self, parent)

        if isinstance(value, RealIntervalAbsoluteElement):
            prec_diff = (<RealIntervalAbsoluteField_class>(<Element>value)._parent)._absprec - parent._absprec
            if prec_diff > 0:
                self._mantissa = shift_floor((<RealIntervalAbsoluteElement>value)._mantissa, prec_diff)
                self._diameter = shift_ceil((<RealIntervalAbsoluteElement>value)._diameter, prec_diff)
            else:
                self._mantissa = (<RealIntervalAbsoluteElement>value)._mantissa << -prec_diff
                self._diameter = (<RealIntervalAbsoluteElement>value)._diameter << -prec_diff
            return

        if isinstance(value, tuple):
            value, upper = value
        elif isinstance(value, RealIntervalFieldElement):
            value, upper = value.lower(), value.upper()
        else:
            upper = None
        value = shift_left(value, parent._absprec)
        if upper is not None:
            upper = shift_left(upper, parent._absprec)
        else:
            upper = value
        from sage.functions.other import floor, ceil
        try:
            self._mantissa = floor(value)
            self._diameter = ceil(upper) - self._mantissa
        except OverflowError:
            raise TypeError(type(value))

    def __reduce__(self):
        """
        Used for pickling.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(50)
            sage: loads(dumps(R(1/16)))
            0.06250000000000000?
            sage: R = RealIntervalAbsoluteField(100)
            sage: loads(dumps(R(1/3)))
            0.333333333333333333333333333334?
            sage: loads(dumps(R(pi))).endpoints() == R(pi).endpoints()
            True
        """
        return RealIntervalAbsoluteElement, (self._parent, self.endpoints())

    cdef _new_c(self, Integer _mantissa, Integer _diameter):
        cdef RealIntervalAbsoluteElement x
        x = <RealIntervalAbsoluteElement>RealIntervalAbsoluteElement.__new__(RealIntervalAbsoluteElement)
        x._parent = self._parent
        x._mantissa = _mantissa
        x._diameter = _diameter
        return x

    cpdef lower(self):
        """
        Return the lower bound of ``self``.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(50)
            sage: R(1/4).lower()
            1/4
        """
        return QQ(self._mantissa) >> (<RealIntervalAbsoluteField_class>self._parent)._absprec

    cpdef midpoint(self):
        """
        Return the midpoint of ``self``.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: R(1/4).midpoint()
            1/4
            sage: R(pi).midpoint()
            7964883625991394727376702227905/2535301200456458802993406410752
            sage: R(pi).midpoint().n()
            3.14159265358979
        """
        return (self._mantissa + self._diameter / 2) >> (<RealIntervalAbsoluteField_class>self._parent)._absprec

    cpdef upper(self):
        """
        Return the upper bound of ``self``.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(50)
            sage: R(1/4).upper()
            1/4
        """
        return QQ(self._mantissa + self._diameter) >> (<RealIntervalAbsoluteField_class>self._parent)._absprec

    cpdef absolute_diameter(self):
        """
        Return the diameter ``self``.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(1/4).absolute_diameter()
            0
            sage: a = R(pi)
            sage: a.absolute_diameter()
            1/1024
            sage: a.upper() - a.lower()
            1/1024
        """
        return QQ(self._diameter) >> (<RealIntervalAbsoluteField_class>self._parent)._absprec

    diameter = absolute_diameter

    cpdef endpoints(self):
        """
        Return the left and right endpoints of ``self``, as a tuple.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(1/4).endpoints()
            (1/4, 1/4)
            sage: R((1,2)).endpoints()
            (1, 2)
        """
        return self.lower(), self.upper()

    def _real_mpfi_(self, R):
        """
        Create a (relative) real interval out of this absolute real interval.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(1/2)._real_mpfi_(RIF)
            0.50000000000000000?

            sage: a = RealIntervalAbsoluteField(100)(1/3)
            sage: RIF(a)
            0.3333333333333334?
        """
        return R(self._mantissa, self._mantissa + self._diameter) >> (<RealIntervalAbsoluteField_class>self._parent)._absprec

    cpdef long mpfi_prec(self) noexcept:
        """
        Return the precision needed to represent this value as an mpfi interval.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(10).mpfi_prec()
            14
            sage: R(1000).mpfi_prec()
            20
        """
        return max(mpz_sizeinbase(self._mantissa.value, 2), mpz_sizeinbase(self._diameter.value, 2))

    def _repr_(self):
        """
        Leverage real interval printing.

        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(1/3) # indirect doctest
            0.334?
            sage: R(10^50/3)
            3.3333333333333333333333333333333333333333333333333334?e49
            sage: R(0)
            0
        """
        prec = max(self.mpfi_prec(), 5)
        return repr(self._real_mpfi_(RealIntervalField(prec)))

    def __hash__(self):
        """
        Hash to the midpoint of the interval.

        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: hash(R(10))
            10
            sage: hash(R((11,13)))
            12
            sage: hash(R(1/4)) == hash(1/4)
            True
            sage: hash(R(pi))
            891658780           # 32-bit
            532995478001132060  # 64-bit
        """
        return hash(self.midpoint())

    def __contains__(self, x):
        """
        Return whether the given value lies in this interval.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(50)
            sage: 1 in R((1,2))
            True
            sage: 2 in R((1,2))
            True
            sage: 3 in R((1,2))
            False
            sage: 1.75 in R((1,2))
            True
        """
        x *= (one << self._parent.absprec())
        return self._mantissa <= x <= self._mantissa + self._diameter

    cpdef bint is_positive(self) noexcept:
        """
        Return whether ``self`` is definitely positive.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(10).is_positive()
            True
            sage: R((10,11)).is_positive()
            True
            sage: R((0,11)).is_positive()
            False
            sage: R((-10,11)).is_positive()
            False
            sage: R((-10,-1)).is_positive()
            False
            sage: R(pi).is_positive()
            True
        """
        return mpz_sgn(self._mantissa.value) == 1

    cpdef bint contains_zero(self) noexcept:
        """
        Return whether ``self`` contains zero.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(10).contains_zero()
            False
            sage: R((10,11)).contains_zero()
            False
            sage: R((0,11)).contains_zero()
            True
            sage: R((-10,11)).contains_zero()
            True
            sage: R((-10,-1)).contains_zero()
            False
            sage: R((-10,0)).contains_zero()
            True
            sage: R(pi).contains_zero()
            False
        """
        return (mpz_sgn(self._mantissa.value) == 0
                or (mpz_sgn(self._mantissa.value) == -1 and mpz_cmpabs(self._mantissa.value, self._diameter.value) <= 0))

    cpdef bint is_negative(self) noexcept:
        """
        Return whether ``self`` is definitely negative.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: R(10).is_negative()
            False
            sage: R((10,11)).is_negative()
            False
            sage: R((0,11)).is_negative()
            False
            sage: R((-10,11)).is_negative()
            False
            sage: R((-10,-1)).is_negative()
            True
            sage: R(pi).is_negative()
            False
        """
        return (mpz_sgn(self._mantissa.value) == -1
                and mpz_cmpabs(self._mantissa.value, self._diameter.value) > 0)

    cdef bint is_exact(self) noexcept:
        return not self._diameter

    def __bool__(self):
        """
        Return ``True`` for anything except exact zero.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: bool(R(1))
            True
            sage: bool(R(0))
            False
            sage: bool(R((0,1)))
            True
        """
        return not not self._mantissa or not not self._diameter

    def __neg__(self):
        """
        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: -R(1/2)
            -0.50000000000000000000000000000000?
            sage: -R((101,102))
            -102.?
        """
        return self._new_c(-self._mantissa - self._diameter, self._diameter)

    def __abs__(self):
        """
        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: abs(-R(1/4))
            0.2500000000000000000000000000000?
        """
        return self.abs()

    cpdef abs(self):
        """
        Return the absolute value of ``self``.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: R(1/3).abs()
            0.333333333333333333333333333334?
            sage: R(-1/3).abs()
            0.333333333333333333333333333334?
            sage: R((-1/3, 1/2)).abs()
            1.?
            sage: R((-1/3, 1/2)).abs().endpoints()
            (0, 1/2)
            sage: R((-3/2, 1/2)).abs().endpoints()
            (0, 3/2)
        """
        if self.is_positive():
            return self
        elif self.is_negative():
            return -self
        else:
            return self._new_c(zero, max(-self._mantissa, self._mantissa + self._diameter))

    cpdef _add_(self, _other):
        """
        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(1) + R(2) # indirect doctest
            3
            sage: R(1e100) + R(0.1) + R(-1e100)
            0.100?
            sage: (R((1,2)) + 1).endpoints()
            (2, 3)
            sage: (R((1,2)) + R((-10,0))).endpoints()
            (-9, 2)
        """
        cdef RealIntervalAbsoluteElement other = <RealIntervalAbsoluteElement>_other
        return self._new_c(self._mantissa + other._mantissa, self._diameter + other._diameter)

    cpdef _sub_(self, _other):
        """
        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(1) - R(2) # indirect doctest
            -1
            sage: R(1e100) - R(0.1) - R(1e100)
            -0.100?
            sage: (R((1,2)) - 1).endpoints()
            (0, 1)
            sage: (R((1,2)) - R((10,100))).endpoints()
            (-99, -8)
            sage: R(pi) - R(pi)
            0.000?
        """
        cdef RealIntervalAbsoluteElement other = <RealIntervalAbsoluteElement>_other
        return self._new_c(self._mantissa - other._mantissa - other._diameter, self._diameter + other._diameter)

    cpdef _mul_(self, _other):
        """
        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(2) * R(3) # indirect doctest
            6
            sage: R(2) * R(-3)
            -6
            sage: elts = [R((left, right)) for left in [-2..2] for right in [left+1..2]]
            sage: elts
            [-2.?, -1.?, 0.?e1, 0.?e1, -1.?, 0.?, 0.?e1, 1.?, 1.?, 2.?]
            sage: for a in elts:
            ....:     for b in elts:
            ....:         if (a*b).lower() != (a._real_mpfi_(RIF)*b._real_mpfi_(RIF)).lower():
            ....:             print(a, b)
            ....:         if (a*b).upper() != (a._real_mpfi_(RIF)*b._real_mpfi_(RIF)).upper():
            ....:             print(a, b)
            sage: R(pi) * R(pi) - R(pi^2)
            0.00?
        """
        cdef bint negate = False
        cdef RealIntervalAbsoluteElement other = <RealIntervalAbsoluteElement>_other
        cdef long absprec = (<RealIntervalAbsoluteField_class>self._parent)._absprec

        # Break symmetry.
        if self.is_negative():
            negate = not negate
            self = -self
        if other.is_negative():
            negate = not negate
            other = -other
        elif other.contains_zero():
            self, other = other, self

        if self.is_positive():
            if other.is_positive():
                res = self._new_c(shift_floor(self._mantissa * other._mantissa, absprec),
                                  shift_ceil(self._mantissa * other._diameter + (other._mantissa + other._diameter) * self._diameter, absprec))
            else:
                res = self._new_c(shift_floor((self._mantissa + self._diameter) * other._mantissa, absprec),
                                  shift_ceil((self._mantissa + self._diameter) * other._diameter, absprec))
        else:
            # They both contain zero.
            self_lower, self_upper = self._mantissa, self._mantissa + self._diameter
            other_lower, other_upper = other._mantissa, other._mantissa + other._diameter
            lower = min(self_lower * other_upper, self_upper * other_lower)
            upper = max(self_lower * other_lower, self_upper * other_upper)
            res = self._new_c(shift_floor(lower, absprec),
                              shift_ceil(upper - lower, absprec))
        if negate:
            res = -res
        return res

    cpdef _acted_upon_(self, x, bint self_on_left):
        """
        ``Absprec * relprec -> absprec`` works better than coercing both
        operands to absolute precision first.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(4)
            sage: 1/4 * R(100)
            25
            sage: 1/100 * R(100)
            1
            sage: R(1/100) * R(100)
            1.?e1
            sage: RIF(1/100) * R(100)
            1.0?

            sage: R(1.5)._acted_upon_(3, True)
            4.500?
        """
        if x < 0:
            neg = self._acted_upon_(-x, self_on_left)
            return None if neg is None else -neg
        if type(x) in (int, Integer):
            return self._new_c(self._mantissa * x, self._diameter * x)
        P = parent_of(x)
        if isinstance(P, Parent):
            if P.is_exact():
                left = (self._mantissa * x).floor()
                right = ((self._mantissa + self._diameter) * x).ceil()
                return self._new_c(left, right - left)
            elif isinstance(x, RealIntervalFieldElement):
                if x.contains_zero() or self.contains_zero():
                    return self * RealIntervalAbsoluteElement(self._parent, (x.lower(), x.upper()))
                # Remember, x > 0
                if self.is_positive():
                    left = (self._mantissa * x.lower()).floor()
                    right = ((self._mantissa + self._diameter) * x.upper()).ceil()
                else:
                    left = (self._mantissa * x.upper()).floor()
                    right = ((self._mantissa + self._diameter) * x.lower()).ceil()
                return self._new_c(left, right - left)

    def __invert__(self):
        """
        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: ~R(2)
            0.50000?
            sage: ~~R(2)
            2
            sage: ~R(3)
            0.334?
            sage: ~~R(3)
            3.00?

            sage: R = RealIntervalAbsoluteField(200)
            sage: ~R(1e10)
            1.00000000000000000000000000000000000000000000000000?e-10
            sage: ~~R(1e10)
            1.00000000000000000000000000000000000000000000000000?e10
            sage: (~R((1,2))).endpoints()
            (1/2, 1)
            sage: (~R((1/4,8))).endpoints()
            (1/8, 4)
            sage: R(1/pi) - 1/R(pi)
            0.?e-60
        """
        if self.contains_zero():
            raise ZeroDivisionError("Inversion of an interval containing zero.")
        cdef long absprec = (<RealIntervalAbsoluteField_class>self._parent)._absprec
        cdef bint negate
        if self.is_negative():
            self = -self
            negate = True
        else:
            negate = False

        # Let our (positive) interval be [2^-B a, 2^-B b].
        # Then its inverse is [2^-B 2^(2B)/b , 2^-B 2^(2B)/a].

        cdef Integer mantissa = <Integer>PY_NEW(Integer)
        cdef Integer diameter = <Integer>PY_NEW(Integer)
        cdef mpz_t scaling_factor
        mpz_init_set_ui(scaling_factor, 1)
        try:
            mpz_set_ui(scaling_factor, 1)
            mpz_mul_2exp(scaling_factor, scaling_factor, 2 * absprec)
            # Use diameter as temp value for right endpoint...
            mpz_add(diameter.value, self._mantissa.value, self._diameter.value)
            mpz_fdiv_q(mantissa.value, scaling_factor, diameter.value)
            # Divide a second time to get the rounding correct...
            mpz_cdiv_q(diameter.value, scaling_factor, self._mantissa.value)
            mpz_sub(diameter.value, diameter.value, mantissa.value)
        finally:
            mpz_clear(scaling_factor)

        res = self._new_c(mantissa, diameter)
        if negate:
            res = -res
        return res

    cpdef _div_(self, _other):
        """
        TESTS::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(5)/R(2) # indirect doctest
            2.5000?
            sage: a = R((5,6))/R((2,4))
            sage: a.endpoints()
            (5/4, 3)
            sage: a = R((-5,6))/R((2,4))
            sage: a.endpoints()
            (-5/2, 3)
            sage: R(1e100) / R(2e100)
            0.50000?
        """
        cdef RealIntervalAbsoluteElement other = <RealIntervalAbsoluteElement>_other
        if other.contains_zero():
            raise ZeroDivisionError("Division by an interval containing zero.")

        cdef Integer mantissa = <Integer>PY_NEW(Integer)
        cdef Integer diameter = <Integer>PY_NEW(Integer)
        cdef long absprec = (<RealIntervalAbsoluteField_class>self._parent)._absprec
        cdef bint negate = False
        cdef mpz_t temp
        mpz_init(temp)

        try:

            if self.is_negative():
                negate = True
                self = -self

            mpz_mul_2exp(mantissa.value, self._mantissa.value, absprec)
            if self.contains_zero():
                mpz_fdiv_q(mantissa.value, mantissa.value, other._mantissa.value)
            else:
                mpz_add(temp, other._mantissa.value, other._diameter.value)
                mpz_fdiv_q(mantissa.value, mantissa.value, temp)

            mpz_add(temp, self._mantissa.value, self._diameter.value)
            mpz_mul_2exp(diameter.value, temp, absprec)
            mpz_cdiv_q(diameter.value, diameter.value, other._mantissa.value)
            mpz_sub(diameter.value, diameter.value, mantissa.value)

        finally:
            mpz_clear(temp)

        res = self._new_c(mantissa, diameter)
        if negate:
            res = -res
        return res

    def __lshift__(RealIntervalAbsoluteElement self, long n):
        """
        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(1) << 2
            4
            sage: R(3) << -2
            0.75000?
            sage: (R((1/2, 5)) << 10).endpoints()
            (512, 5120)
        """
        return self.shift(n)

    def __rshift__(RealIntervalAbsoluteElement self, long n):
        """
        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(1) >> 2
            0.2500?
            sage: R(3) >> -2
            12
            sage: (R((1/2, 5)) >> 10).endpoints()
            (0, 5/1024)
        """
        return self.shift(-n)

    cdef shift(self, long n):
        if n >= 0:
            return self._new_c(self._mantissa << n, self._diameter << n)
        else:
            return self._new_c(shift_floor(self._mantissa, -n), shift_ceil(self._diameter, -n))

    def __pow__(self, exponent, dummy):
        """
        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(10)
            sage: R(10)^10
            10000000000
            sage: R(10)^100
            10000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
            sage: R(10)^-2
            0.010?
            sage: R(10)^-10
            0.001?
            sage: R(100)^(1/2)
            10.00?
            sage: (R((2,3))^2).endpoints()
            (4, 9)
            sage: (R((-5,3))^3).endpoints()
            (-125, 75)
            sage: (R((-5,3))^4).endpoints()
            (0, 625)
        """
        cdef RealIntervalAbsoluteElement base
        try:
            base = self
        except TypeError:
            base = exponent.parent()(self)
        cdef RealIntervalAbsoluteField_class parent = self.parent()
        if not base:
            if not exponent:
                raise ZeroDivisionError
            else:
                return base
        import math
        cdef double height
        base_height = max(abs(base.lower()), abs(base.upper()))
        height = math.log(base_height) / math.log(2)
        height *= <double>(exponent.midpoint() if isinstance(exponent, RealIntervalAbsoluteElement) else exponent)
        relprec = max(<long>height + parent._absprec, 10)
        RIF = RealIntervalField(relprec)
        if isinstance(exponent, RealIntervalAbsoluteElement):
            exponent = exponent._real_mpfi_(RIF)
        return parent(base._real_mpfi_(RIF)**exponent)

    def __getattr__(self, name):
        """
        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: R(1).sin()
            0.841470984807896506652502321631?
            sage: R(2).log()
            0.693147180559945309417232121458?
            sage: R(1).exp().log()
            1.00000000000000000000000000000?

            sage: R((0,10)).sin().endpoints()
            (-1, 1)
            sage: R(1).sin().parent()
            Real Interval Field with absolute precision 2^-100
        """
        if name[0] != '_' and hasattr(RealIntervalFieldElement, name):
            return MpfrOp(self, name)
        else:
            raise AttributeError(name)

    def sqrt(self):
        """
        Return the square root of ``self``.

        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
            sage: R = RealIntervalAbsoluteField(100)
            sage: R(2).sqrt()
            1.414213562373095048801688724210?
            sage: R((4,9)).sqrt().endpoints()
            (2, 3)
        """
        return self._parent(self._real_mpfi_(RealIntervalField(self.mpfi_prec())).sqrt())

cdef class MpfrOp:
    """
    This class is used to endow absolute real interval field elements with
    all the methods of (relative) real interval field elements.

    EXAMPLES::

        sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField
        sage: R = RealIntervalAbsoluteField(100)
        sage: R(1).sin()
        0.841470984807896506652502321631?
    """
    cdef object name
    cdef RealIntervalAbsoluteElement value

    def __init__(self, value, name):
        """
        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField, MpfrOp
            sage: R = RealIntervalAbsoluteField(100)
            sage: MpfrOp(R(1), 'tan')()
            1.557407724654902230506974807459?
        """
        self.name = name
        self.value = value

    def __call__(self, *args):
        """
        EXAMPLES::

            sage: from sage.rings.real_interval_absolute import RealIntervalAbsoluteField, MpfrOp
            sage: R = RealIntervalAbsoluteField(100)
            sage: curried_log = MpfrOp(R(2), 'log')
            sage: curried_log()
            0.693147180559945309417232121458?
            sage: curried_log(2)
            1
        """
        cdef RealIntervalAbsoluteField_class parent = self.value._parent
        cdef long absprec = parent._absprec
        cdef long relprec = self.value.mpfi_prec()
        for a in args:
            if isinstance(a, RealIntervalAbsoluteElement):
                absprec = min(absprec, (<RealIntervalAbsoluteField_class>(<RealIntervalAbsoluteElement>a)._parent)._absprec)
                relprec = max(absprec, (<RealIntervalAbsoluteElement>a).mpfi_prec())
        if parent._absprec > absprec:
            parent = RealIntervalAbsoluteField(absprec)
        if relprec < 53:
            relprec = 53
        R = RealIntervalField(relprec)
        new_args = []
        for a in args:
            if isinstance(a, RealIntervalAbsoluteElement):
                new_args.append(a._real_mpfi_(R))
            else:
                new_args.append(a)
        return parent(getattr(self.value._real_mpfi_(R), self.name)(*new_args))



# sage_setup: distribution = sagemath-modules
r"""
Huffman encoding

This module implements functionalities relating to Huffman encoding and
decoding.

AUTHOR:

- Nathann Cohen (2010-05): initial version.

Classes and functions
=====================
"""

###########################################################################
# Copyright (c) 2010 Nathann Cohen <nathann.cohen@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# https://www.gnu.org/licenses/
###########################################################################

from collections import defaultdict

from sage.structure.sage_object import SageObject


###########################################################################
#
# Helper functions
#
###########################################################################

def frequency_table(string):
    r"""
    Return the frequency table corresponding to the given string.

    INPUT:

    - ``string`` -- string of symbols over some alphabet

    OUTPUT:

    - A table of frequency of each unique symbol in ``string``. If ``string``
      is an empty string, return an empty table.

    EXAMPLES:

    The frequency table of a non-empty string::

        sage: from sage.coding.source_coding.huffman import frequency_table
        sage: str = "Stop counting my characters!"
        sage: T = sorted(frequency_table(str).items())
        sage: for symbol, code in T:
        ....:     print("{} {}".format(symbol, code))
          3
        ! 1
        S 1
        a 2
        c 3
        e 1
        g 1
        h 1
        i 1
        m 1
        n 2
        o 2
        p 1
        r 2
        s 1
        t 3
        u 1
        y 1

    The frequency of an empty string::

        sage: frequency_table("")
        defaultdict(<... 'int'>, {})
    """
    d = defaultdict(int)
    for s in string:
        d[s] += 1
    return d


class Huffman(SageObject):
    r"""
    This class implements the basic functionalities of Huffman codes.

    It can build a Huffman code from a given string, or from the information
    of a dictionary associating to each key (the elements of the alphabet) a
    weight (most of the time, a probability value or a number of occurrences).

    INPUT:

    - ``source`` -- can be either

      - A string from which the Huffman encoding should be created.

      - A dictionary that associates to each symbol of an alphabet a numeric
        value. If we consider the frequency of each alphabetic symbol, then
        ``source`` is considered as the frequency table of the alphabet with
        each numeric (nonnegative integer) value being the number of
        occurrences of a symbol. The numeric values can also represent weights
        of the symbols. In that case, the numeric values are not necessarily
        integers, but can be real numbers.

    In order to construct a Huffman code for an alphabet, we use exactly one of
    the following methods:

    #. Let ``source`` be a string of symbols over an alphabet and feed
       ``source`` to the constructor of this class. Based on the input string, a
       frequency table is constructed that contains the frequency of each unique
       symbol in ``source``. The alphabet in question is then all the unique
       symbols in ``source``. A significant implication of this is that any
       subsequent string that we want to encode must contain only symbols that
       can be found in ``source``.

    #. Let ``source`` be the frequency table of an alphabet. We can feed this
       table to the constructor of this class. The table ``source`` can be a
       table of frequencies or a table of weights.

    In either case, the alphabet must consist of at least two symbols.

    EXAMPLES::

        sage: from sage.coding.source_coding.huffman import Huffman, frequency_table
        sage: h1 = Huffman("There once was a french fry")
        sage: for letter, code in sorted(h1.encoding_table().items()):
        ....:     print("'{}' : {}".format(letter, code))
        ' ' : 00
        'T' : 11100
        'a' : 0111
        'c' : 1010
        'e' : 100
        'f' : 1011
        'h' : 1100
        'n' : 1101
        'o' : 11101
        'r' : 010
        's' : 11110
        'w' : 11111
        'y' : 0110

    We can obtain the same result by "training" the Huffman code with the
    following table of frequency::

        sage: ft = frequency_table("There once was a french fry")
        sage: sorted(ft.items())
        [(' ', 5),
         ('T', 1),
         ('a', 2),
         ('c', 2),
         ('e', 4),
         ('f', 2),
         ('h', 2),
         ('n', 2),
         ('o', 1),
         ('r', 3),
         ('s', 1),
         ('w', 1),
         ('y', 1)]

        sage: h2 = Huffman(ft)

    Once ``h1`` has been trained, and hence possesses an encoding table,
    it is possible to obtain the Huffman encoding of any string
    (possibly the same) using this code::

        sage: encoded = h1.encode("There once was a french fry"); encoded
        '11100110010001010000111011101101010000111110111111100001110010110101001101101011000010110100110'

    We can decode the above encoded string in the following way::

        sage: h1.decode(encoded)
        'There once was a french fry'

    Obviously, if we try to decode a string using a Huffman instance which
    has been trained on a different sample (and hence has a different encoding
    table), we are likely to get some random-looking string::

        sage: h3 = Huffman("There once were two french fries")
        sage: h3.decode(encoded)
        ' eierhffcoeft TfewrnwrTrsc'

    This does not look like our original string.

    Instead of using frequency, we can assign weights to each alphabetic
    symbol::

        sage: from sage.coding.source_coding.huffman import Huffman
        sage: T = {"a":45, "b":13, "c":12, "d":16, "e":9, "f":5}
        sage: H = Huffman(T)
        sage: L = ["deaf", "bead", "fab", "bee"]
        sage: E = []
        sage: for e in L:
        ....:     E.append(H.encode(e))
        ....:     print(E[-1])
        111110101100
        10111010111
        11000101
        10111011101
        sage: D = []
        sage: for e in E:
        ....:     D.append(H.decode(e))
        ....:     print(D[-1])
        deaf
        bead
        fab
        bee
        sage: D == L
        True
    """

    def __init__(self, source):
        r"""
        Constructor for Huffman.

        See the docstring of this class for full documentation.

        EXAMPLES::

            sage: from sage.coding.source_coding.huffman import Huffman
            sage: str = "Sage is my most favorite general purpose computer algebra system"
            sage: h = Huffman(str)

        TESTS:

        Feeding anything else than a string or a dictionary::

            sage: Huffman(Graph())                                                      # needs sage.graphs
            Traceback (most recent call last):
            ...
            ValueError: Input must be either a string or a dictionary.
        """

        # alphabetic symbol to Huffman encoding translation table
        self._character_to_code = []
        # Huffman binary tree
        self._tree = None
        # index of each alphabetic symbol
        self._index = None

        if isinstance(source, str):
            self._build_code(frequency_table(source))
        elif isinstance(source, dict):
            self._build_code(source)
        else:
            raise ValueError("Input must be either a string or a dictionary.")

    def _build_code_from_tree(self, tree, d, prefix):
        r"""
        Build the Huffman code corresponding to a given tree and prefix.

        INPUT:

        - ``tree`` -- integer, or list of size `2`

        - ``d`` -- the dictionary to fill

        - ``prefix`` -- string; binary string which is the prefix
          of any element of the tree

        EXAMPLES::

            sage: from sage.coding.source_coding.huffman import Huffman
            sage: str = "Sage is my most favorite general purpose computer algebra system"
            sage: h = Huffman(str)
            sage: d = {}
            sage: h._build_code_from_tree(h._tree, d, prefix="")
        """
        # This is really a recursive construction of a Huffman code. By
        # feeding this class a sufficiently large alphabet, it is possible to
        # exceed the maximum recursion depth and hence result in a RuntimeError.
        try:
            self._build_code_from_tree(tree[0],
                                       d,
                                       prefix="".join([prefix, "0"]))
            self._build_code_from_tree(tree[1],
                                       d,
                                       prefix="".join([prefix, "1"]))
        except TypeError:
            d[tree] = prefix

    def _build_code(self, dic):
        r"""
        Construct a Huffman code corresponding to an alphabet with the given
        weight table.

        INPUT:

        - ``dic`` -- dictionary that associates to each symbol of an alphabet
          a numeric value. If we consider the frequency of each alphabetic
          symbol, then ``dic`` is considered as the frequency table of the
          alphabet with each numeric (nonnegative integer) value being the
          number of occurrences of a symbol. The numeric values can also
          represent weights of the symbols. In that case, the numeric values
          are not necessarily integers, but can be real numbers. In general,
          we refer to ``dic`` as a weight table.

        EXAMPLES::

            sage: from sage.coding.source_coding.huffman import Huffman, frequency_table
            sage: str = "Sage is my most favorite general purpose computer algebra system"
            sage: h = Huffman(str)
            sage: h._build_code(frequency_table(str))

        TESTS:

        Trying to build a Huffman code for fewer than two symbols fails.  We
        could support these corner cases of course, but instead we just don't
        allow it::

            sage: Huffman('')
            Traceback (most recent call last):
            ...
            ValueError: The alphabet for Huffman must contain at least two
            symbols.
            sage: Huffman(' ')
            Traceback (most recent call last):
            ...
            ValueError: The alphabet for Huffman must contain at least two
            symbols.
        """

        if len(dic) < 2:
            # Most of the rest of this class assumes there are at least two
            # symbols in the alphabet; we could explicitly handle the corner
            # cases of 0 or 1 characters, but they are also pretty useless so
            # enforce 2 or more characters
            raise ValueError(
                "The alphabet for {} must contain at least two symbols.".format(
                    self.__class__.__name__))

        symbols = sorted(dic.items(), key=lambda x: (x[1], x[0]))

        # Each alphabetic symbol is now represented by an element with weight w
        # and index i.
        q0 = [(weight, idx) for idx, (sym, weight) in enumerate(symbols)]
        q1 = []

        queues = [q0, q1]
        tot_weight = sum(s[1] for s in symbols)  # Just used as a sentinel below

        def pop():
            # pop the lowest weight node from the heads of the two queues (as
            # long as at least one of them has one node)
            q = min(queues, key=lambda q: (q and q[0][0] or tot_weight))
            return q.pop(0)

        while len(q0) + len(q1) > 1:
            weight_a, node_a = pop()
            weight_b, node_b = pop()
            q1.append((weight_a + weight_b, (node_a, node_b)))

        # dictionary of symbol to Huffman encoding
        d = {}
        self._tree = q1[0][1]
        # Build the binary tree of a Huffman code, where the root of the tree
        # is associated with the empty string.
        self._build_code_from_tree(self._tree, d, prefix="")
        self._index = {i: s for i, (s, w) in enumerate(symbols)}
        self._character_to_code = {
            s: d[i] for i, (s, w) in enumerate(symbols)}

    def encode(self, string):
        r"""
        Encode the given string based on the current encoding table.

        INPUT:

        - ``string`` -- string of symbols over an alphabet

        OUTPUT: a Huffman encoding of ``string``

        EXAMPLES:

        This is how a string is encoded and then decoded::

            sage: from sage.coding.source_coding.huffman import Huffman
            sage: str = "Sage is my most favorite general purpose computer algebra system"
            sage: h = Huffman(str)
            sage: encoded = h.encode(str); encoded
            '11000011010001010101100001111101001110011101001101101111011110111001111010000101101110100000111010101000101000000010111011011000110100101001011100010011011110101011100100110001100101001001110101110101110110001000101011000111101101101111110011111101110100011'
            sage: h.decode(encoded)
            'Sage is my most favorite general purpose computer algebra system'
        """
        if self._character_to_code:
            return "".join(self._character_to_code[x] for x in string)

    def decode(self, string):
        r"""
        Decode the given string using the current encoding table.

        INPUT:

        - ``string`` -- string of Huffman encodings

        OUTPUT: the Huffman decoding of ``string``

        EXAMPLES:

        This is how a string is encoded and then decoded::

            sage: from sage.coding.source_coding.huffman import Huffman
            sage: str = "Sage is my most favorite general purpose computer algebra system"
            sage: h = Huffman(str)
            sage: encoded = h.encode(str); encoded
            '11000011010001010101100001111101001110011101001101101111011110111001111010000101101110100000111010101000101000000010111011011000110100101001011100010011011110101011100100110001100101001001110101110101110110001000101011000111101101101111110011111101110100011'
            sage: h.decode(encoded)
            'Sage is my most favorite general purpose computer algebra system'

        TESTS:

        Of course, the string one tries to decode has to be a binary one. If
        not, an exception is raised::

            sage: h.decode('I clearly am not a binary string')
            Traceback (most recent call last):
            ...
            ValueError: Input must be a binary string.
        """
        # This traverses the whole Huffman binary tree in order to work out
        # the symbol represented by a stream of binaries. This method of
        # decoding is really slow. A faster method is needed.
        # TODO: faster decoding implementation
        chars = []
        tree = self._tree
        index = self._index
        for i in string:
            if i == "0":
                tree = tree[0]
            elif i == "1":
                tree = tree[1]
            else:
                raise ValueError("Input must be a binary string.")
            if not isinstance(tree, tuple):
                chars.append(index[tree])
                tree = self._tree
        return "".join(chars)

    def encoding_table(self):
        r"""
        Return the current encoding table.

        INPUT:

        - None.

        OUTPUT: a dictionary associating an alphabetic symbol to a Huffman encoding

        EXAMPLES::

            sage: from sage.coding.source_coding.huffman import Huffman
            sage: str = "Sage is my most favorite general purpose computer algebra system"
            sage: h = Huffman(str)
            sage: T = sorted(h.encoding_table().items())
            sage: for symbol, code in T:
            ....:     print("{} {}".format(symbol, code))
              101
            S 110000
            a 1101
            b 110001
            c 110010
            e 010
            f 110011
            g 0001
            i 10000
            l 10001
            m 0011
            n 00000
            o 0110
            p 0010
            r 1110
            s 1111
            t 0111
            u 10010
            v 00001
            y 10011
        """
        return self._character_to_code.copy()

    def tree(self):
        r"""
        Return the Huffman tree corresponding to the current encoding.

        INPUT:

        - None.

        OUTPUT: the binary tree representing a Huffman code

        EXAMPLES::

            sage: from sage.coding.source_coding.huffman import Huffman
            sage: str = "Sage is my most favorite general purpose computer algebra system"
            sage: h = Huffman(str)
            sage: T = h.tree(); T                                                       # needs sage.graphs
            Digraph on 39 vertices
            sage: T.show(figsize=[20,20])                                               # needs sage.graphs sage.plot
            <BLANKLINE>
        """
        from sage.graphs.digraph import DiGraph
        g = DiGraph()
        g.add_edges(self._generate_edges(self._tree))
        return g

    def _generate_edges(self, tree, parent='', bit=''):
        """
        Generate the edges of the given Huffman tree.

        INPUT:

        - ``tree`` -- a Huffman binary tree

        - ``parent`` -- (default: empty string) a parent vertex with exactly
          two children

        - ``bit`` -- (default: empty string) the bit signifying either the
          left or right branch; the bit '0' denotes the left branch and '1'
          denotes the right branch

        OUTPUT: an edge list of the Huffman binary tree

        EXAMPLES::

            sage: from sage.coding.source_coding.huffman import Huffman
            sage: H = Huffman("Sage")
            sage: T = H.tree()                                                          # needs sage.graphs
            sage: T.edges(sort=True, labels=None)  # indirect doctest                   # needs sage.graphs
            [('0', 'S: 00'), ('0', 'a: 01'), ('1', 'e: 10'), ('1', 'g: 11'), ('root', '0'), ('root', '1')]
        """
        u = "root" if parent == "" else parent
        s = "".join([parent, bit])
        try:
            left = self._generate_edges(tree[0], parent=s, bit='0')
            right = self._generate_edges(tree[1], parent=s, bit='1')
            L = [(u, s)] if s else []
            return left + right + L
        except TypeError:
            return [(u, "".join([self.decode(s), ": ", s]))]



# sage_setup: distribution = sagemath-flint
# Macros from fmpz_poly.h
# See https://github.com/flintlib/flint/issues/152

from .types cimport *

cdef extern from "flint_wrap.h":
    fmpq_ptr fmpq_poly_get_coeff_ptr(fmpq_poly_t p, ulong n)
    # Macro giving a pointer to the n-th coefficient of p (or NULL)



from sage.libs.all__sagemath_combinat import *
from sage.libs.all__sagemath_gap import *
from sage.libs.all__sagemath_flint import *
from sage.libs.all__sagemath_ntl import *
from sage.libs.all__sagemath_pari import *
from sage.libs.all__sagemath_eclib import *

try:
    from sage.libs.all__sagemath_symbolics import *
except ImportError:
    pass
